# encoding: utf-8

"""Common shape-related oxml objects."""

from __future__ import absolute_import, division, print_function, unicode_literals

from pptx.dml.fill import CT_GradientFillProperties
from pptx.enum.shapes import PP_PLACEHOLDER
from pptx.oxml.ns import qn
from pptx.oxml.simpletypes import (
    ST_Angle,
    ST_Coordinate,
    ST_Direction,
    ST_DrawingElementId,
    ST_LineWidth,
    ST_PlaceholderSize,
    ST_PositiveCoordinate,
    XsdBoolean,
    XsdString,
    XsdUnsignedInt,
)
from pptx.oxml.xmlchemy import (
    BaseOxmlElement,
    Choice,
    OptionalAttribute,
    OxmlElement,
    RequiredAttribute,
    ZeroOrOne,
    ZeroOrOneChoice,
)
from pptx.util import Emu


class BaseShapeElement(BaseOxmlElement):
    """
    Provides common behavior for shape element classes like CT_Shape,
    CT_Picture, etc.
    """

    @property
    def cx(self):
        return self._get_xfrm_attr("cx")

    @cx.setter
    def cx(self, value):
        self._set_xfrm_attr("cx", value)

    @property
    def cy(self):
        return self._get_xfrm_attr("cy")

    @cy.setter
    def cy(self, value):
        self._set_xfrm_attr("cy", value)

    @property
    def flipH(self):
        return bool(self._get_xfrm_attr("flipH"))

    @flipH.setter
    def flipH(self, value):
        self._set_xfrm_attr("flipH", value)

    @property
    def flipV(self):
        return bool(self._get_xfrm_attr("flipV"))

    @flipV.setter
    def flipV(self, value):
        self._set_xfrm_attr("flipV", value)

    def get_or_add_xfrm(self):
        """
        Return the ``<a:xfrm>`` grandchild element, newly-added if not
        present. This version works for ``<p:sp>``, ``<p:cxnSp>``, and
        ``<p:pic>`` elements, others will need to override.
        """
        return self.spPr.get_or_add_xfrm()

    @property
    def has_ph_elm(self):
        """
        True if this shape element has a ``<p:ph>`` descendant, indicating it
        is a placeholder shape. False otherwise.
        """
        return self.ph is not None

    @property
    def ph(self):
        """
        The ``<p:ph>`` descendant element if there is one, None otherwise.
        """
        ph_elms = self.xpath("./*[1]/p:nvPr/p:ph")
        if len(ph_elms) == 0:
            return None
        return ph_elms[0]

    @property
    def ph_idx(self):
        """
        Integer value of placeholder idx attribute. Raises |ValueError| if
        shape is not a placeholder.
        """
        ph = self.ph
        if ph is None:
            raise ValueError("not a placeholder shape")
        return ph.idx

    @property
    def ph_orient(self):
        """
        Placeholder orientation, e.g. 'vert'. Raises |ValueError| if shape is
        not a placeholder.
        """
        ph = self.ph
        if ph is None:
            raise ValueError("not a placeholder shape")
        return ph.orient

    @property
    def ph_sz(self):
        """
        Placeholder size, e.g. ST_PlaceholderSize.HALF, None if shape has no
        ``<p:ph>`` descendant.
        """
        ph = self.ph
        if ph is None:
            raise ValueError("not a placeholder shape")
        return ph.sz

    @property
    def ph_type(self):
        """
        Placeholder type, e.g. ST_PlaceholderType.TITLE ('title'), none if
        shape has no ``<p:ph>`` descendant.
        """
        ph = self.ph
        if ph is None:
            raise ValueError("not a placeholder shape")
        return ph.type

    @property
    def rot(self):
        """
        Float representing degrees this shape is rotated clockwise.
        """
        xfrm = self.xfrm
        if xfrm is None:
            return 0.0
        return xfrm.rot

    @rot.setter
    def rot(self, value):
        self.get_or_add_xfrm().rot = value

    @property
    def shape_id(self):
        """
        Integer id of this shape
        """
        return self._nvXxPr.cNvPr.id

    @property
    def shape_name(self):
        """
        Name of this shape
        """
        return self._nvXxPr.cNvPr.name

    @property
    def txBody(self):
        """
        Child ``<p:txBody>`` element, None if not present
        """
        return self.find(qn("p:txBody"))

    @property
    def x(self):
        return self._get_xfrm_attr("x")

    @x.setter
    def x(self, value):
        self._set_xfrm_attr("x", value)

    @property
    def xfrm(self):
        """
        The ``<a:xfrm>`` grandchild element or |None| if not found. This
        version works for ``<p:sp>``, ``<p:cxnSp>``, and ``<p:pic>``
        elements, others will need to override.
        """
        return self.spPr.xfrm

    @property
    def y(self):
        return self._get_xfrm_attr("y")

    @y.setter
    def y(self, value):
        self._set_xfrm_attr("y", value)

    @property
    def _nvXxPr(self):
        """
        Required non-visual shape properties element for this shape. Actual
        name depends on the shape type, e.g. ``<p:nvPicPr>`` for picture
        shape.
        """
        return self.xpath("./*[1]")[0]

    def _get_xfrm_attr(self, name):
        xfrm = self.xfrm
        if xfrm is None:
            return None
        return getattr(xfrm, name)

    def _set_xfrm_attr(self, name, value):
        xfrm = self.get_or_add_xfrm()
        setattr(xfrm, name, value)


class CT_ApplicationNonVisualDrawingProps(BaseOxmlElement):
    """
    ``<p:nvPr>`` element
    """

    ph = ZeroOrOne(
        "p:ph",
        successors=(
            "a:audioCd",
            "a:wavAudioFile",
            "a:audioFile",
            "a:videoFile",
            "a:quickTimeFile",
            "p:custDataLst",
            "p:extLst",
        ),
    )


class CT_LineProperties(BaseOxmlElement):
    """Custom element class for <a:ln> element"""

    _tag_seq = (
        "a:noFill",
        "a:solidFill",
        "a:gradFill",
        "a:pattFill",
        "a:prstDash",
        "a:custDash",
        "a:round",
        "a:bevel",
        "a:miter",
        "a:headEnd",
        "a:tailEnd",
        "a:extLst",
    )
    eg_lineFillProperties = ZeroOrOneChoice(
        (
            Choice("a:noFill"),
            Choice("a:solidFill"),
            Choice("a:gradFill"),
            Choice("a:pattFill"),
        ),
        successors=_tag_seq[4:],
    )
    prstDash = ZeroOrOne("a:prstDash", successors=_tag_seq[5:])
    custDash = ZeroOrOne("a:custDash", successors=_tag_seq[6:])
    del _tag_seq
    w = OptionalAttribute("w", ST_LineWidth, default=Emu(0))

    @property
    def eg_fillProperties(self):
        """
        Required to fulfill the interface used by dml.fill.
        """
        return self.eg_lineFillProperties

    @property
    def prstDash_val(self):
        """Return value of `val` attribute of `a:prstDash` child.

        Return |None| if not present.
        """
        prstDash = self.prstDash
        if prstDash is None:
            return None
        return prstDash.val

    @prstDash_val.setter
    def prstDash_val(self, val):
        self._remove_custDash()
        prstDash = self.get_or_add_prstDash()
        prstDash.val = val


class CT_NonVisualDrawingProps(BaseOxmlElement):
    """
    ``<p:cNvPr>`` custom element class.
    """

    _tag_seq = ("a:hlinkClick", "a:hlinkHover", "a:extLst")
    hlinkClick = ZeroOrOne("a:hlinkClick", successors=_tag_seq[1:])
    hlinkHover = ZeroOrOne("a:hlinkHover", successors=_tag_seq[2:])
    id = RequiredAttribute("id", ST_DrawingElementId)
    name = RequiredAttribute("name", XsdString)
    del _tag_seq


class CT_Placeholder(BaseOxmlElement):
    """
    ``<p:ph>`` custom element class.
    """

    type = OptionalAttribute("type", PP_PLACEHOLDER, default=PP_PLACEHOLDER.OBJECT)
    orient = OptionalAttribute("orient", ST_Direction, default=ST_Direction.HORZ)
    sz = OptionalAttribute("sz", ST_PlaceholderSize, default=ST_PlaceholderSize.FULL)
    idx = OptionalAttribute("idx", XsdUnsignedInt, default=0)


class CT_Point2D(BaseOxmlElement):
    """
    Custom element class for <a:off> element.
    """

    x = RequiredAttribute("x", ST_Coordinate)
    y = RequiredAttribute("y", ST_Coordinate)


class CT_PositiveSize2D(BaseOxmlElement):
    """
    Custom element class for <a:ext> element.
    """

    cx = RequiredAttribute("cx", ST_PositiveCoordinate)
    cy = RequiredAttribute("cy", ST_PositiveCoordinate)


class CT_ShapeProperties(BaseOxmlElement):
    """Custom element class for `p:spPr` element.

    Shared by `p:sp`, `p:cxnSp`,  and `p:pic` elements as well as a few more
    obscure ones.
    """

    _tag_seq = (
        "a:xfrm",
        "a:custGeom",
        "a:prstGeom",
        "a:noFill",
        "a:solidFill",
        "a:gradFill",
        "a:blipFill",
        "a:pattFill",
        "a:grpFill",
        "a:ln",
        "a:effectLst",
        "a:effectDag",
        "a:scene3d",
        "a:sp3d",
        "a:extLst",
    )
    xfrm = ZeroOrOne("a:xfrm", successors=_tag_seq[1:])
    custGeom = ZeroOrOne("a:custGeom", successors=_tag_seq[2:])
    prstGeom = ZeroOrOne("a:prstGeom", successors=_tag_seq[3:])
    eg_fillProperties = ZeroOrOneChoice(
        (
            Choice("a:noFill"),
            Choice("a:solidFill"),
            Choice("a:gradFill"),
            Choice("a:blipFill"),
            Choice("a:pattFill"),
            Choice("a:grpFill"),
        ),
        successors=_tag_seq[9:],
    )
    ln = ZeroOrOne("a:ln", successors=_tag_seq[10:])
    effectLst = ZeroOrOne("a:effectLst", successors=_tag_seq[11:])
    del _tag_seq

    @property
    def cx(self):
        """
        Shape width as an instance of Emu, or None if not present.
        """
        cx_str_lst = self.xpath("./a:xfrm/a:ext/@cx")
        if not cx_str_lst:
            return None
        return Emu(cx_str_lst[0])

    @property
    def cy(self):
        """
        Shape height as an instance of Emu, or None if not present.
        """
        cy_str_lst = self.xpath("./a:xfrm/a:ext/@cy")
        if not cy_str_lst:
            return None
        return Emu(cy_str_lst[0])

    @property
    def x(self):
        """
        The offset of the left edge of the shape from the left edge of the
        slide, as an instance of Emu. Corresponds to the value of the
        `./xfrm/off/@x` attribute. None if not present.
        """
        x_str_lst = self.xpath("./a:xfrm/a:off/@x")
        if not x_str_lst:
            return None
        return Emu(x_str_lst[0])

    @property
    def y(self):
        """
        The offset of the top of the shape from the top of the slide, as an
        instance of Emu. None if not present.
        """
        y_str_lst = self.xpath("./a:xfrm/a:off/@y")
        if not y_str_lst:
            return None
        return Emu(y_str_lst[0])

    def _new_gradFill(self):
        return CT_GradientFillProperties.new_gradFill()


class CT_Transform2D(BaseOxmlElement):
    """`a:xfrm` custom element class.

    NOTE: this is a composite including CT_GroupTransform2D, which appears
    with the `a:xfrm` tag in a group shape (including a slide `p:spTree`).
    """

    _tag_seq = ("a:off", "a:ext", "a:chOff", "a:chExt")
    off = ZeroOrOne("a:off", successors=_tag_seq[1:])
    ext = ZeroOrOne("a:ext", successors=_tag_seq[2:])
    chOff = ZeroOrOne("a:chOff", successors=_tag_seq[3:])
    chExt = ZeroOrOne("a:chExt", successors=_tag_seq[4:])
    del _tag_seq
    rot = OptionalAttribute("rot", ST_Angle, default=0.0)
    flipH = OptionalAttribute("flipH", XsdBoolean, default=False)
    flipV = OptionalAttribute("flipV", XsdBoolean, default=False)

    @property
    def x(self):
        off = self.off
        if off is None:
            return None
        return off.x

    @x.setter
    def x(self, value):
        off = self.get_or_add_off()
        off.x = value

    @property
    def y(self):
        off = self.off
        if off is None:
            return None
        return off.y

    @y.setter
    def y(self, value):
        off = self.get_or_add_off()
        off.y = value

    @property
    def cx(self):
        ext = self.ext
        if ext is None:
            return None
        return ext.cx

    @cx.setter
    def cx(self, value):
        ext = self.get_or_add_ext()
        ext.cx = value

    @property
    def cy(self):
        ext = self.ext
        if ext is None:
            return None
        return ext.cy

    @cy.setter
    def cy(self, value):
        ext = self.get_or_add_ext()
        ext.cy = value

    def _new_ext(self):
        ext = OxmlElement("a:ext")
        ext.cx = 0
        ext.cy = 0
        return ext

    def _new_off(self):
        off = OxmlElement("a:off")
        off.x = 0
        off.y = 0
        return off



# encoding: utf-8

"""
Objects related to layout of rendered text, such as TextFitter.
"""

from __future__ import absolute_import, print_function

from PIL import ImageFont


class TextFitter(tuple):
    """
    Value object that knows how to fit text into given rectangular extents.
    """

    def __new__(cls, line_source, extents, font_file):
        width, height = extents
        return tuple.__new__(cls, (line_source, width, height, font_file))

    @classmethod
    def best_fit_font_size(cls, text, extents, max_size, font_file):
        """
        Return the largest whole-number point size less than or equal to
        *max_size* that allows *text* to fit completely within *extents* when
        rendered using font defined in *font_file*.
        """
        line_source = _LineSource(text)
        text_fitter = cls(line_source, extents, font_file)
        return text_fitter._best_fit_font_size(max_size)

    def _best_fit_font_size(self, max_size):
        """
        Return the largest whole-number point size less than or equal to
        *max_size* that this fitter can fit.
        """
        predicate = self._fits_inside_predicate
        sizes = _BinarySearchTree.from_ordered_sequence(range(1, int(max_size) + 1))
        return sizes.find_max(predicate)

    def _break_line(self, line_source, point_size):
        """
        Return a (line, remainder) pair where *line* is the longest line in
        *line_source* that will fit in this fitter's width and *remainder* is
        a |_LineSource| object containing the text following the break point.
        """
        lines = _BinarySearchTree.from_ordered_sequence(line_source)
        predicate = self._fits_in_width_predicate(point_size)
        return lines.find_max(predicate)

    def _fits_in_width_predicate(self, point_size):
        """
        Return a function taking a text string value and returns |True| if
        that text fits in this fitter when rendered at *point_size*. Used as
        predicate for _break_line()
        """

        def predicate(line):
            """
            Return |True| if *line* fits in this fitter when rendered at
            *point_size*.
            """
            cx = _rendered_size(line.text, point_size, self._font_file)[0]
            return cx <= self._width

        return predicate

    @property
    def _fits_inside_predicate(self):
        """
        Return a function taking an integer point size argument that returns
        |True| if the text in this fitter can be wrapped to fit entirely
        within its extents when rendered at that point size.
        """

        def predicate(point_size):
            """
            Return |True| if the text in *line_source* can be wrapped to fit
            entirely within *extents* when rendered at *point_size* using the
            font defined in *font_file*.
            """
            text_lines = self._wrap_lines(self._line_source, point_size)
            cy = _rendered_size("Ty", point_size, self._font_file)[1]
            return (cy * len(text_lines)) <= self._height

        return predicate

    @property
    def _font_file(self):
        return self[3]

    @property
    def _height(self):
        return self[2]

    @property
    def _line_source(self):
        return self[0]

    @property
    def _width(self):
        return self[1]

    def _wrap_lines(self, line_source, point_size):
        """
        Return a sequence of str values representing the text in
        *line_source* wrapped within this fitter when rendered at
        *point_size*.
        """
        text, remainder = self._break_line(line_source, point_size)
        lines = [text]
        if remainder:
            lines.extend(self._wrap_lines(remainder, point_size))
        return lines


class _BinarySearchTree(object):
    """
    A node in a binary search tree. Uniform for root, subtree root, and leaf
    nodes.
    """

    def __init__(self, value):
        self._value = value
        self._lesser = None
        self._greater = None

    def find_max(self, predicate, max_=None):
        """
        Return the largest item in or under this node that satisfies
        *predicate*.
        """
        if predicate(self.value):
            max_ = self.value
            next_node = self._greater
        else:
            next_node = self._lesser
        if next_node is None:
            return max_
        return next_node.find_max(predicate, max_)

    @classmethod
    def from_ordered_sequence(cls, iseq):
        """
        Return the root of a balanced binary search tree populated with the
        values in iterable *iseq*.
        """
        seq = list(iseq)
        # optimize for usually all fits by making longest first
        bst = cls(seq.pop())
        bst._insert_from_ordered_sequence(seq)
        return bst

    def insert(self, value):
        """
        Insert a new node containing *value* into this tree such that its
        structure as a binary search tree is preserved.
        """
        side = "_lesser" if value < self.value else "_greater"
        child = getattr(self, side)
        if child is None:
            setattr(self, side, _BinarySearchTree(value))
        else:
            child.insert(value)

    def tree(self, level=0, prefix=""):
        """
        A string representation of the tree rooted in this node, useful for
        debugging purposes.
        """
        text = "%s%s\n" % (prefix, self.value.text)
        prefix = "%s└── " % ("    " * level)
        if self._lesser:
            text += self._lesser.tree(level + 1, prefix)
        if self._greater:
            text += self._greater.tree(level + 1, prefix)
        return text

    @property
    def value(self):
        """
        The value object contained in this node.
        """
        return self._value

    @staticmethod
    def _bisect(seq):
        """
        Return a (medial_value, greater_values, lesser_values) 3-tuple
        obtained by bisecting sequence *seq*.
        """
        if len(seq) == 0:
            return [], None, []
        mid_idx = int(len(seq) / 2)
        mid = seq[mid_idx]
        greater = seq[mid_idx + 1 :]
        lesser = seq[:mid_idx]
        return mid, greater, lesser

    def _insert_from_ordered_sequence(self, seq):
        """
        Insert the new values contained in *seq* into this tree such that
        a balanced tree is produced.
        """
        if len(seq) == 0:
            return
        mid, greater, lesser = self._bisect(seq)
        self.insert(mid)
        self._insert_from_ordered_sequence(greater)
        self._insert_from_ordered_sequence(lesser)


class _LineSource(object):
    """
    Generates all the possible even-word line breaks in a string of text,
    each in the form of a (line, remainder) 2-tuple where *line* contains the
    text before the break and *remainder* the text after as a |_LineSource|
    object. Its boolean value is |True| when it contains text, |False| when
    its text is the empty string or whitespace only.
    """

    def __init__(self, text):
        self._text = text

    def __bool__(self):
        """
        Gives this object boolean behaviors (in Python 3). bool(line_source)
        is False if it contains the empty string or whitespace only.
        """
        return self._text.strip() != ""

    def __eq__(self, other):
        return self._text == other._text

    def __iter__(self):
        """
        Generate a (text, remainder) pair for each possible even-word line
        break in this line source, where *text* is a str value and remainder
        is a |_LineSource| value.
        """
        words = self._text.split()
        for idx in range(1, len(words) + 1):
            line_text = " ".join(words[:idx])
            remainder_text = " ".join(words[idx:])
            remainder = _LineSource(remainder_text)
            yield _Line(line_text, remainder)

    def __nonzero__(self):
        """
        Gives this object boolean behaviors (in Python 2). bool(line_source)
        is False if it contains the empty string or whitespace only.
        """
        return self._text.strip() != ""

    def __repr__(self):
        return "<_LineSource('%s')>" % self._text


class _Line(tuple):
    """
    A candidate line broken at an even word boundary from a string of text,
    and a |_LineSource| value containing the text that remains after the line
    is broken at this spot.
    """

    def __new__(cls, text, remainder):
        return tuple.__new__(cls, (text, remainder))

    def __gt__(self, other):
        return len(self.text) > len(other.text)

    def __lt__(self, other):
        return not self.__gt__(other)

    def __len__(self):
        return len(self.text)

    def __repr__(self):
        return "'%s' => '%s'" % (self.text, self.remainder)

    @property
    def remainder(self):
        return self[1]

    @property
    def text(self):
        return self[0]


class _Fonts(object):
    """
    A memoizing cache for ImageFont objects.
    """

    fonts = {}

    @classmethod
    def font(cls, font_path, point_size):
        if (font_path, point_size) not in cls.fonts:
            cls.fonts[(font_path, point_size)] = ImageFont.truetype(
                font_path, point_size
            )
        return cls.fonts[(font_path, point_size)]


def _rendered_size(text, point_size, font_file):
    """
    Return a (width, height) pair representing the size of *text* in English
    Metric Units (EMU) when rendered at *point_size* in the font defined in
    *font_file*.
    """
    emu_per_inch = 914400
    px_per_inch = 72.0

    font = _Fonts.font(font_file, point_size)
    px_width, px_height = font.getsize(text)

    emu_width = int(px_width / px_per_inch * emu_per_inch)
    emu_height = int(px_height / px_per_inch * emu_per_inch)

    return emu_width, emu_height



# encoding: utf-8

"""Gherkin step implementations for ShadowFormat-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then, when

from pptx import Presentation

from helpers import test_pptx


# given ====================================================


@given("a ShadowFormat object that {inherits} as shadow")
def given_a_ShadowFormat_object_that_inherits_or_not(context, inherits):
    shape_idx = {"inherits": 0, "does not inherit": 1}[inherits]
    shape = Presentation(test_pptx("dml-effect")).slides[0].shapes[shape_idx]
    context.shadow = shape.shadow


# when =====================================================


@when("I assign {value} to shadow.inherit")
def when_I_assign_value_to_shadow_inherit(context, value):
    context.shadow.inherit = eval(value)


# then =====================================================


@then("shadow.inherit is {bool_str}")
def then_shadow_inherit_is_bool_val(context, bool_str):
    expected_value = eval(bool_str)
    actual_value = context.shadow.inherit
    assert actual_value is expected_value, "shadow.inherit is %s" % actual_value



# encoding: utf-8

"""Gherkin step implementations for shape collections."""

import io

from behave import given, then, when

from pptx import Presentation
from pptx.chart.data import CategoryChartData
from pptx.enum.chart import XL_CHART_TYPE
from pptx.enum.shapes import MSO_CONNECTOR, MSO_SHAPE, PP_PLACEHOLDER, PROG_ID
from pptx.shapes.base import BaseShape
from pptx.util import Emu, Inches

from helpers import saved_pptx_path, test_file, test_image, test_pptx


# given ===================================================


@given("a _BaseShapes object as shapes")
def given_a_BaseShapes_object_as_shapes(context):
    prs = Presentation()
    context.shapes = prs.slides.add_slide(prs.slide_layouts[6]).shapes


@given("a GroupShapes object as shapes")
@given("a GroupShapes object of length 3 as shapes")
def given_a_GroupShapes_object_of_length_3_as_shapes(context):
    prs = Presentation(test_pptx("shp-groupshape"))
    group_shape = prs.slides[0].shapes[0]
    context.shapes = group_shape.shapes


@given("a LayoutPlaceholders object of length 2 as shapes")
def given_a_LayoutPlaceholders_object_of_length_2_as_shapes(context):
    prs = Presentation(test_pptx("lyt-shapes"))
    context.shapes = prs.slide_layouts[0].placeholders


@given("a LayoutShapes object of length 3 as shapes")
def given_a_LayoutShapes_object_of_length_3_as_shapes(context):
    prs = Presentation(test_pptx("lyt-shapes"))
    context.shapes = prs.slide_layouts[0].shapes


@given("a MasterPlaceholders object of length 2 as shapes")
def given_a_MasterPlaceholders_object_of_length_2_as_shapes(context):
    prs = Presentation(test_pptx("mst-placeholders"))
    context.shapes = prs.slide_masters[0].placeholders


@given("a MasterShapes object of length 2 as shapes")
def given_a_MasterShapes_object_of_length_2_as_shapes(context):
    prs = Presentation(test_pptx("mst-shapes"))
    context.shapes = prs.slide_masters[0].shapes


@given("a {PROG_ID_member} file as ole_object_file")
def given_a_PROG_ID_member_file_as_ole_object_file(context, PROG_ID_member):
    filename = {
        "DOCX": "shp-embedded-docx.docx",
        "PPTX": "shp-embedded-pptx.pptx",
        "XLSX": "shp-embedded-xlsx.xlsx",
    }[PROG_ID_member]
    with open(test_file(filename), "rb") as f:
        context.ole_object_file = io.BytesIO(f.read())
    context.PROG_ID_member = PROG_ID_member


@given("a SlidePlaceholders object of length 2 as shapes")
def given_a_SlidePlaceholders_object_of_length_2_as_shapes(context):
    prs = Presentation(test_pptx("shp-shapes"))
    context.shapes = prs.slides[0].placeholders


@given("a SlideShapes object as shapes")
def given_a_SlideShapes_object_as_shapes(context):
    prs = Presentation(test_pptx("shp-shapes"))
    context.shapes = prs.slides[0].shapes


@given("a SlideShapes object containing {a_or_no} movies")
def given_a_SlideShapes_object_containing_a_or_no_movies(context, a_or_no):
    pptx = {"one or more": "shp-movie-props", "no": "shp-shapes"}[a_or_no]
    prs = Presentation(test_pptx(pptx))
    context.prs = prs
    context.shapes = prs.slides[0].shapes


@given("a SlideShapes object of length 6 shapes as shapes")
def given_a_SlideShapes_object_of_length_6_as_shapes(context):
    prs = Presentation(test_pptx("shp-shapes"))
    context.shapes = prs.slides[0].shapes


@given("a SlideShapes object having a {type} shape at offset {idx}")
def given_a_SlideShapes_obj_having_type_shape_at_off_idx(context, type, idx):
    prs = Presentation(test_pptx("shp-shapes"))
    context.shapes = prs.slides[1].shapes


# when ====================================================


@when("I add 100 shapes")
def when_I_add_100_shapes(context):
    X_ORIG = Y_ORIG = Inches(0.0625)
    X_INCR = Y_INCR = Inches(0.5)
    CX = CY = Inches(0.375)

    def iter_corner():
        y = Y_ORIG
        while True:
            for i in range(20):
                x = X_ORIG + (X_INCR * i)
                yield x, y
            y += Y_INCR

    shapes = context.shapes
    corners = iter_corner()
    for i in range(100):
        x, y = next(corners)
        shapes.add_shape(MSO_SHAPE.ROUNDED_RECTANGLE, x, y, CX, CY)


@when("I add a table to the slide's shape collection")
def when_I_call_shapes_add_table(context):
    shapes = context.slide.shapes
    x, y = (Inches(1.00), Inches(2.00))
    cx, cy = (Inches(3.00), Inches(1.00))
    shapes.add_table(2, 2, x, y, cx, cy)


@when("I assign shape.ole_format to ole_format")
def when_I_assign_shape_ole_format_to_ole_format(context):
    context.ole_format = context.shape.ole_format


@when("I assign shapes.add_chart() to shape")
def when_I_assign_shapes_add_chart_to_shape(context):
    chart_data = CategoryChartData()
    chart_data.categories = ("Foo", "Bar")
    chart_data.add_series("East", (1.0, 2.0))
    chart_data.add_series("West", (3.0, 4.0))

    context.shape = context.shapes.add_chart(
        XL_CHART_TYPE.COLUMN_CLUSTERED,
        Inches(1),
        Inches(1),
        Inches(8),
        Inches(5),
        chart_data,
    )


@when("I assign shapes.add_connector() to shape")
def when_I_assign_shapes_add_connector_to_shape(context):
    context.shape = context.shapes.add_connector(MSO_CONNECTOR.CURVE, 4, 3, 2, 1)


@when("I assign shapes.add_group_shape() to shape")
def when_I_assign_shapes_add_group_shape_to_shape(context):
    context.shape = context.shapes.add_group_shape()


@when("I assign shapes.add_ole_object(ole_object_file) to shape")
def when_I_assign_shapes_add_ole_object_to_shape(context):
    context.shape = context.shapes.add_ole_object(
        context.ole_object_file, getattr(PROG_ID, context.PROG_ID_member), 4, 3, 2, 1
    )


@when("I assign shapes.add_picture() to shape")
def when_I_assign_shapes_add_picture_to_shape(context):
    context.shape = context.shapes.add_picture(
        test_image("sonic.gif"), Inches(1), Inches(2)
    )


@when("I assign shapes.add_shape() to shape")
def when_I_assign_shapes_add_shape_to_shape(context):
    context.shape = context.shapes.add_shape(
        MSO_SHAPE.ROUNDED_RECTANGLE, Inches(2), Inches(3), Inches(1), Inches(0.5)
    )


@when("I assign shapes.add_textbox() to shape")
def when_I_assign_shapes_add_textbox_to_shape(context):
    context.shape = context.shapes.add_textbox(
        Inches(1), Inches(2), Inches(3), Inches(0.5)
    )


@when("I assign shapes.build_freeform() to builder")
def when_I_assign_shapes_build_freeform_to_builder(context):
    shapes = context.shapes
    builder = shapes.build_freeform()
    context.builder = builder


@when("I assign shapes.build_freeform(scale=100.0) to builder")
def when_I_assign_shapes_build_freeform_scale_to_builder(context):
    shapes = context.shapes
    builder = shapes.build_freeform(scale=100.0)
    context.builder = builder


@when("I assign shapes.build_freeform(scale=(200.0, 100.0)) to builder")
def when_I_assign_shapes_build_freeform_scale_rectnglr_to_builder(context):
    shapes = context.shapes
    builder = shapes.build_freeform(scale=(200.0, 100.0))
    context.builder = builder


@when("I assign shapes.build_freeform(start_x=25, start_y=125) to builder")
def when_I_assign_shapes_build_freeform_start_x_start_y_to_builder(context):
    shapes = context.shapes
    builder = shapes.build_freeform(25, 125)
    context.builder = builder


@when("I assign True to shapes.turbo_add_enabled")
def when_I_assign_True_to_shapes_turbo_add_enabled(context):
    context.shapes.turbo_add_enabled = True


@when("I call shapes.add_chart({type_}, chart_data)")
def when_I_call_shapes_add_chart(context, type_):
    chart_type = getattr(XL_CHART_TYPE, type_)
    context.chart = context.shapes.add_chart(
        chart_type, 0, 0, 0, 0, context.chart_data
    ).chart


@when("I call shapes.add_connector(MSO_CONNECTOR.STRAIGHT, 1, 2, 3, 4)")
def when_I_call_shapes_add_connector(context):
    context.connector = context.shapes.add_connector(MSO_CONNECTOR.STRAIGHT, 1, 2, 3, 4)


@when("I call shapes.add_movie(file, x, y, cx, cy, poster_frame)")
def when_I_call_shapes_add_movie(context):
    shapes = context.shapes
    x, y, cx, cy = Emu(2590800), Emu(571500), Emu(3962400), Emu(5715000)
    context.movie = shapes.add_movie(
        test_file("just-two-mice.mp4"), x, y, cx, cy, test_file("just-two-mice.png")
    )


# then ====================================================


@then("iterating shapes produces {count} objects of type {class_name}")
def then_iterating_shapes_produces_count_objects_of_type_class_name(
    context, count, class_name
):
    shapes = context.shapes
    expected_count, expected_class_name = int(count), class_name
    idx = -1
    for idx, shape in enumerate(shapes):
        actual_class_name = shape.__class__.__name__
        assert actual_class_name == expected_class_name, (
            "shape.__class__.__name__ == %s" % actual_class_name
        )
    actual_count = idx + 1
    assert actual_count == expected_count, "got %d items" % actual_count


@then("iterating shapes produces {count} objects that subclass BaseShape")
def then_iterating_shapes_produces_count_objects_that_subclass_BaseShape(
    context, count
):
    shapes = context.shapes
    expected_count = int(count)
    idx = -1
    for idx, shape in enumerate(shapes):
        class_name = shape.__class__.__name__
        assert isinstance(shape, BaseShape), (
            "%s does not subclass BaseShape" % class_name
        )
    actual_count = idx + 1
    assert actual_count == expected_count, "got %d items" % actual_count


@then("len(shapes) == {value}")
def then_len_shapes_eq_value(context, value):
    expected_len = int(value)
    actual_len = len(context.shapes)
    assert actual_len == expected_len, "len(shapes) == %s" % actual_len


@then("shape is a {clsname} object")
def then_shape_is_a_type_object(context, clsname):
    actual_class_name = context.shape.__class__.__name__
    expected_class_name = clsname
    assert actual_class_name == expected_class_name, (
        "shape is a %s object" % actual_class_name
    )


@then("shapes[-1] == shape")
def then_shapes_minus_1_eq_shape(context):
    shapes, shape = context.shapes, context.shape
    assert shapes[-1] == shape


@then("shapes[{idx}] is a {type_} object")
def then_shapes_idx_is_a_type_object(context, idx, type_):
    shapes = context.shapes
    type_name = type(shapes[int(idx)]).__name__
    assert type_name == type_, "got %s" % type_name


@then("shapes.get(idx=10) is the body placeholder")
def then_shapes_get_10_is_the_body_placeholder(context):
    shapes = context.shapes
    title_placeholder = shapes.get(idx=0)
    body_placeholder = shapes.get(idx=10)
    assert title_placeholder._element is shapes[0]._element
    assert body_placeholder._element is shapes[1]._element


@then("shapes.get(PP_PLACEHOLDER.BODY) is the body placeholder")
def then_shapes_get_PP_PLACEHOLDER_BODY_is_the_body_ph(context):
    shapes = context.shapes
    title_placeholder = shapes.get(PP_PLACEHOLDER.TITLE)
    body_placeholder = shapes.get(PP_PLACEHOLDER.BODY)
    assert title_placeholder._element is shapes[0]._element
    assert body_placeholder._element is shapes[1]._element


@then("shapes.index(shape) for each shape matches its sequence position")
def then_shapes_index_for_each_shape_matches_sequence_position(context):
    shapes = context.shapes
    for idx, shape in enumerate(shapes):
        assert idx == shapes.index(shape), "index doesn't match for idx == %s" % idx


@then("shapes.title is the title placeholder")
def then_shapes_title_is_the_title_placeholder(context):
    shapes = context.shapes
    title_placeholder = shapes.title
    assert title_placeholder.element is shapes[0].element
    assert title_placeholder.shape_id == 4


@then("shapes.turbo_add_enabled is False")
def then_shapes_turbo_add_enabled_is_False(context):
    shapes = context.shapes
    assert shapes.turbo_add_enabled is False


@then("the table appears in the slide")
def then_the_table_appears_in_the_slide(context):
    prs = Presentation(saved_pptx_path)
    expected_table_graphic_frame = prs.slides[0].shapes[0]
    assert expected_table_graphic_frame.has_table



# encoding: utf-8

"""
Unit test suite for the pptx.chart.category module.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.chart.category import Categories, Category, CategoryLevel
from pptx.oxml import parse_xml

from ..unitutil.cxml import element
from ..unitutil.file import snippet_seq
from ..unitutil.mock import call, class_mock, instance_mock


class DescribeCategories(object):
    def it_knows_its_length(self, len_fixture):
        categories, expected_len = len_fixture
        assert len(categories) == expected_len

    def it_supports_indexed_access(self, getitem_fixture):
        categories, idx, Category_, pt, category_ = getitem_fixture
        category = categories[idx]
        Category_.assert_called_once_with(pt, idx)
        assert category is category_

    def it_can_iterate_over_the_categories_it_contains(self, iter_fixture):
        categories, expected_categories, Category_, calls, = iter_fixture
        assert [c for c in categories] == expected_categories
        assert Category_.call_args_list == calls

    def it_knows_its_depth(self, depth_fixture):
        categories, expected_value = depth_fixture
        assert categories.depth == expected_value

    def it_provides_a_flattened_representation(self, flat_fixture):
        categories, expected_values = flat_fixture
        flattened_labels = categories.flattened_labels
        assert flattened_labels == expected_values

    def it_provides_access_to_its_levels(self, levels_fixture):
        categories, CategoryLevel_, calls, expected_levels = levels_fixture
        levels = categories.levels
        assert CategoryLevel_.call_args_list == calls
        assert levels == expected_levels

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:barChart", 0),
            ("c:barChart/c:ser/c:cat", 1),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/c:lvl", 1),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/(c:lvl,c:lvl)", 2),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/(c:lvl,c:lvl,c:lvl)", 3),
        ]
    )
    def depth_fixture(self, request):
        xChart_cxml, expected_value = request.param
        categories = Categories(element(xChart_cxml))
        return categories, expected_value

    @pytest.fixture(
        params=[
            # non-hierarchical (no lvls), 3 categories, one "none"
            (0, (("Foo",), ("",), ("Baz",))),
            # 2-lvls
            (1, (("CA", "SF"), ("CA", "LA"), ("NY", "NY"), ("NY", "Albany"))),
            # 3-lvls
            (
                2,
                (
                    ("USA", "CA", "SF"),
                    ("USA", "CA", "LA"),
                    ("USA", "NY", "NY"),
                    ("USA", "NY", "Albany"),
                    ("CAN", "AL", "Calgary"),
                    ("CAN", "AL", "Edmonton"),
                    ("CAN", "ON", "Toronto"),
                    ("CAN", "ON", "Ottawa"),
                ),
            ),
            # 1-lvl; not seen in wild but spec does not prohibit
            (3, (("SF",), ("LA",), ("NY",))),
        ]
    )
    def flat_fixture(self, request):
        snippet_idx, expected_values = request.param
        xChart_xml = snippet_seq("cat-labels")[snippet_idx]
        categories = Categories(parse_xml(xChart_xml))
        return categories, expected_values

    @pytest.fixture(
        params=[
            ("c:barChart/c:ser/c:cat/(c:ptCount{val=2})", 0, None),
            ("c:barChart/c:ser/c:cat/(c:ptCount{val=2},c:pt{idx=1})", 0, None),
            ("c:barChart/c:ser/c:cat/(c:ptCount{val=2},c:pt{idx=1})", 1, 0),
            ("c:barChart/c:ser/c:cat/c:lvl/(c:ptCount{val=2},c:pt{idx=1})", 1, 0),
        ]
    )
    def getitem_fixture(self, request, Category_, category_):
        xChart_cxml, idx, pt_offset = request.param
        xChart = element(xChart_cxml)
        pt = None if pt_offset is None else xChart.xpath(".//c:pt")[pt_offset]
        categories = Categories(xChart)
        return categories, idx, Category_, pt, category_

    @pytest.fixture
    def iter_fixture(self, Category_, category_):
        xChart = element("c:barChart/c:ser/c:cat/(c:ptCount{val=2},c:pt{idx=1})")
        pt = xChart.xpath(".//c:pt")[0]
        categories = Categories(xChart)
        expected_categories = [category_, category_]
        calls = [call(None, 0), call(pt, 1)]
        return categories, expected_categories, Category_, calls

    @pytest.fixture(
        params=[("c:barChart", 0), ("c:barChart/c:ser/c:cat/c:ptCount{val=4}", 4)]
    )
    def len_fixture(self, request):
        xChart_cxml, expected_len = request.param
        categories = Categories(element(xChart_cxml))
        return categories, expected_len

    @pytest.fixture(
        params=[
            ("c:barChart", 0),
            ("c:barChart/c:ser/c:cat", 0),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/c:lvl", 1),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/(c:lvl,c:lvl)", 2),
            ("c:barChart/c:ser/c:cat/c:multiLvlStrRef/(c:lvl,c:lvl,c:lvl)", 3),
        ]
    )
    def levels_fixture(self, request, CategoryLevel_, category_level_):
        xChart_cxml, level_count = request.param
        xChart = element(xChart_cxml)
        lvls = xChart.xpath(".//c:lvl")
        categories = Categories(xChart)
        calls = [call(lvl) for lvl in lvls]
        expected_levels = [category_level_ for _ in range(level_count)]
        return categories, CategoryLevel_, calls, expected_levels

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Category_(self, request, category_):
        return class_mock(
            request, "pptx.chart.category.Category", return_value=category_
        )

    @pytest.fixture
    def category_(self, request):
        return instance_mock(request, Category)

    @pytest.fixture
    def CategoryLevel_(self, request, category_level_):
        return class_mock(
            request, "pptx.chart.category.CategoryLevel", return_value=category_level_
        )

    @pytest.fixture
    def category_level_(self, request):
        return instance_mock(request, CategoryLevel)


class DescribeCategory(object):
    def it_extends_str(self, base_class_fixture):
        category, str_value = base_class_fixture
        assert isinstance(category, str)
        assert category == str_value

    def it_knows_its_idx(self, idx_fixture):
        category, expected_value = idx_fixture
        idx = category.idx
        assert idx == expected_value

    def it_knows_its_label(self, label_fixture):
        category, expected_value = label_fixture
        label = category.label
        assert label == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[(None, ""), ('c:pt/c:v"Foo"', "Foo")])
    def base_class_fixture(self, request):
        pt_cxml, str_value = request.param
        pt = None if pt_cxml is None else element(pt_cxml)
        category = Category(pt)
        return category, str_value

    @pytest.fixture(params=[(None, 42, 42), ('c:pt{idx=9}/c:v"Bar"', None, 9)])
    def idx_fixture(self, request):
        pt_cxml, idx, expected_value = request.param
        pt = None if pt_cxml is None else element(pt_cxml)
        category = Category(pt, idx)
        return category, expected_value

    @pytest.fixture(params=[(None, ""), ('c:pt{idx=9}/c:v"Bar"', "Bar")])
    def label_fixture(self, request):
        pt_cxml, expected_value = request.param
        pt = None if pt_cxml is None else element(pt_cxml)
        category = Category(pt)
        return category, expected_value


class DescribeCategoryLevel(object):
    def it_knows_its_length(self, len_fixture):
        category_level, expected_len = len_fixture
        assert len(category_level) == expected_len

    def it_supports_indexed_access(self, getitem_fixture):
        category_level, idx, Category_, pt, category_ = getitem_fixture
        category = category_level[idx]
        Category_.assert_called_once_with(pt)
        assert category is category_

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[0, 1, 2])
    def getitem_fixture(self, request, Category_, category_):
        idx = request.param
        lvl_cxml = "c:lvl/(c:pt,c:pt,c:pt)"
        lvl = element(lvl_cxml)
        pt = lvl.xpath("./c:pt")[idx]
        category_level = CategoryLevel(lvl)
        return category_level, idx, Category_, pt, category_

    @pytest.fixture(
        params=[
            ("c:lvl", 0),
            ("c:lvl/c:pt", 1),
            ("c:lvl/(c:pt,c:pt)", 2),
            ("c:lvl/(c:pt,c:pt,c:pt)", 3),
        ]
    )
    def len_fixture(self, request):
        lvl_cxml, expected_len = request.param
        category_level = CategoryLevel(element(lvl_cxml))
        return category_level, expected_len

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Category_(self, request, category_):
        return class_mock(
            request, "pptx.chart.category.Category", return_value=category_
        )

    @pytest.fixture
    def category_(self, request):
        return instance_mock(request, Category)



# encoding: utf-8

"""Unit-test suite for pptx.shapes.freeform module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.shapes.autoshape import Shape
from pptx.shapes.freeform import (
    _BaseDrawingOperation,
    _Close,
    FreeformBuilder,
    _LineSegment,
    _MoveTo,
)
from pptx.shapes.shapetree import SlideShapes

from ..unitutil.cxml import element, xml
from ..unitutil.file import snippet_seq
from ..unitutil.mock import (
    call,
    initializer_mock,
    instance_mock,
    method_mock,
    property_mock,
)


class DescribeFreeformBuilder(object):
    def it_provides_a_constructor(self, new_fixture):
        shapes_, start_x, start_y, x_scale, y_scale = new_fixture[:5]
        _init_, start_x_int, start_y_int = new_fixture[5:]

        builder = FreeformBuilder.new(shapes_, start_x, start_y, x_scale, y_scale)

        _init_.assert_called_once_with(
            builder, shapes_, start_x_int, start_y_int, x_scale, y_scale
        )
        assert isinstance(builder, FreeformBuilder)

    def it_can_add_straight_line_segments(self, add_segs_fixture):
        builder, vertices, close, add_calls, close_calls = add_segs_fixture

        return_value = builder.add_line_segments(vertices, close)

        assert builder._add_line_segment.call_args_list == add_calls
        assert builder._add_close.call_args_list == close_calls
        assert return_value is builder

    def it_can_move_the_pen_location(self, move_to_fixture):
        builder, x, y, _MoveTo_new_, move_to_ = move_to_fixture

        return_value = builder.move_to(x, y)

        _MoveTo_new_.assert_called_once_with(builder, x, y)
        assert builder._drawing_operations[-1] is move_to_
        assert return_value is builder

    def it_can_build_the_specified_freeform_shape(self, convert_fixture):
        builder, origin_x, origin_y, sp = convert_fixture[:4]
        apply_operation_to_, calls, shape_ = convert_fixture[4:]

        shape = builder.convert_to_shape(origin_x, origin_y)

        builder._add_freeform_sp.assert_called_once_with(builder, origin_x, origin_y)
        builder._start_path.assert_called_once_with(builder, sp)
        assert apply_operation_to_.call_args_list == calls
        builder._shapes._shape_factory.assert_called_once_with(sp)
        assert shape is shape_

    def it_knows_the_shape_x_offset(self, shape_offset_x_fixture):
        builder, expected_value = shape_offset_x_fixture
        x_offset = builder.shape_offset_x
        assert x_offset == expected_value

    def it_knows_the_shape_y_offset(self, shape_offset_y_fixture):
        builder, expected_value = shape_offset_y_fixture
        y_offset = builder.shape_offset_y
        assert y_offset == expected_value

    def it_adds_a_freeform_sp_to_help(self, sp_fixture):
        builder, origin_x, origin_y, spTree, expected_xml = sp_fixture

        sp = builder._add_freeform_sp(origin_x, origin_y)

        assert spTree.xml == expected_xml
        assert sp is spTree.xpath("p:sp")[0]

    def it_adds_a_line_segment_to_help(self, add_seg_fixture):
        builder, x, y, _LineSegment_new_, line_segment_ = add_seg_fixture

        builder._add_line_segment(x, y)

        _LineSegment_new_.assert_called_once_with(builder, x, y)
        assert builder._drawing_operations == [line_segment_]

    def it_closes_a_contour_to_help(self, add_close_fixture):
        builder, _Close_new_, close_ = add_close_fixture

        builder._add_close()

        _Close_new_.assert_called_once_with()
        assert builder._drawing_operations == [close_]

    def it_knows_the_freeform_left_extent_to_help(self, left_fixture):
        builder, expected_value = left_fixture
        left = builder._left
        assert left == expected_value

    def it_knows_the_freeform_top_extent_to_help(self, top_fixture):
        builder, expected_value = top_fixture
        top = builder._top
        assert top == expected_value

    def it_knows_the_freeform_width_to_help(self, width_fixture):
        builder, expected_value = width_fixture
        width = builder._width
        assert width == expected_value

    def it_knows_the_freeform_height_to_help(self, height_fixture):
        builder, expected_value = height_fixture
        height = builder._height
        assert height == expected_value

    def it_knows_the_local_coordinate_width_to_help(self, dx_fixture):
        builder, expected_value = dx_fixture
        dx = builder._dx
        assert dx == expected_value

    def it_knows_the_local_coordinate_height_to_help(self, dy_fixture):
        builder, expected_value = dy_fixture
        dy = builder._dy
        assert dy == expected_value

    def it_can_start_a_new_path_to_help(self, start_path_fixture):
        builder, sp, _local_to_shape_ = start_path_fixture[:3]
        start_x, start_y, expected_xml = start_path_fixture[3:]

        path = builder._start_path(sp)

        _local_to_shape_.assert_called_once_with(start_x, start_y)
        assert sp.xml == expected_xml
        assert path is sp.xpath(".//a:path")[-1]

    def it_translates_local_to_shape_coordinates_to_help(self, local_fixture):
        builder, local_x, local_y, expected_value = local_fixture
        shape_x_y = builder._local_to_shape(local_x, local_y)
        assert shape_x_y == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_close_fixture(self, _Close_new_, close_):
        _Close_new_.return_value = close_
        builder = FreeformBuilder(None, None, None, None, None)
        return builder, _Close_new_, close_

    @pytest.fixture
    def add_seg_fixture(self, _LineSegment_new_, line_segment_):
        x, y = 4, 2
        _LineSegment_new_.return_value = line_segment_

        builder = FreeformBuilder(None, None, None, None, None)
        return builder, x, y, _LineSegment_new_, line_segment_

    @pytest.fixture(params=[(True, [call()]), (False, [])])
    def add_segs_fixture(self, request, _add_line_segment_, _add_close_):
        close, close_calls = request.param
        vertices = ((1, 2), (3, 4), (5, 6))
        builder = FreeformBuilder(None, None, None, None, None)
        add_calls = [call(1, 2), call(3, 4), call(5, 6)]
        return builder, vertices, close, add_calls, close_calls

    @pytest.fixture
    def convert_fixture(
        self, shapes_, apply_operation_to_, _add_freeform_sp_, _start_path_, shape_
    ):
        origin_x, origin_y = 42, 24
        sp, path = element("p:sp"), element("a:path")
        drawing_ops = (
            _BaseDrawingOperation(None, None, None),
            _BaseDrawingOperation(None, None, None),
        )
        shapes_._shape_factory.return_value = shape_
        _add_freeform_sp_.return_value = sp
        _start_path_.return_value = path

        builder = FreeformBuilder(shapes_, None, None, None, None)
        builder._drawing_operations.extend(drawing_ops)
        calls = [call(drawing_ops[0], path), call(drawing_ops[1], path)]
        return (builder, origin_x, origin_y, sp, apply_operation_to_, calls, shape_)

    @pytest.fixture(
        params=[
            (0, (1, None, 2, 3), 3),
            (6, (1, None, 2, 3), 5),
            (50, (150, -5, None, 100), 155),
        ]
    )
    def dx_fixture(self, request):
        start_x, xs, expected_value = request.param
        drawing_ops = []
        for x in xs:
            if x is None:
                drawing_ops.append(_Close())
            else:
                drawing_ops.append(_BaseDrawingOperation(None, x, None))

        builder = FreeformBuilder(None, start_x, None, None, None)
        builder._drawing_operations.extend(drawing_ops)
        return builder, expected_value

    @pytest.fixture(
        params=[
            (0, (1, None, 2, 3), 3),
            (2, (1, None, 2, 3), 2),
            (32, (160, -8, None, 101), 168),
        ]
    )
    def dy_fixture(self, request):
        start_y, ys, expected_value = request.param
        drawing_ops = []
        for y in ys:
            if y is None:
                drawing_ops.append(_Close())
            else:
                drawing_ops.append(_BaseDrawingOperation(None, None, y))

        builder = FreeformBuilder(None, None, start_y, None, None)
        builder._drawing_operations.extend(drawing_ops)
        return builder, expected_value

    @pytest.fixture(params=[(0, 2.0, 0), (24, 10.0, 240), (914400, 314.1, 287213040)])
    def height_fixture(self, request, _dy_prop_):
        dy, y_scale, expected_value = request.param
        _dy_prop_.return_value = dy

        builder = FreeformBuilder(None, None, None, None, y_scale)
        return builder, expected_value

    @pytest.fixture(params=[(0, 1.0, 0), (4, 10.0, 40), (914400, 914.3, 836035920)])
    def left_fixture(self, request, shape_offset_x_prop_):
        offset_x, x_scale, expected_value = request.param
        shape_offset_x_prop_.return_value = offset_x

        builder = FreeformBuilder(None, None, None, x_scale, None)
        return builder, expected_value

    @pytest.fixture
    def local_fixture(self, shape_offset_x_prop_, shape_offset_y_prop_):
        local_x, local_y = 123, 456
        shape_offset_x_prop_.return_value = 23
        shape_offset_y_prop_.return_value = 156

        builder = FreeformBuilder(None, None, None, None, None)
        expected_value = (100, 300)
        return builder, local_x, local_y, expected_value

    @pytest.fixture
    def move_to_fixture(self, _MoveTo_new_, move_to_):
        x, y = 42, 24
        _MoveTo_new_.return_value = move_to_

        builder = FreeformBuilder(None, None, None, None, None)
        return builder, x, y, _MoveTo_new_, move_to_

    @pytest.fixture
    def new_fixture(self, shapes_, _init_):
        start_x, start_y, x_scale, y_scale = 99.56, 200.49, 4.2, 2.4
        start_x_int, start_y_int = 100, 200
        return (
            shapes_,
            start_x,
            start_y,
            x_scale,
            y_scale,
            _init_,
            start_x_int,
            start_y_int,
        )

    @pytest.fixture(
        params=[
            (0, (1, None, 2, 3), 0),
            (6, (1, None, 2, 3), 1),
            (50, (150, -5, None, 100), -5),
        ]
    )
    def shape_offset_x_fixture(self, request):
        start_x, xs, expected_value = request.param
        drawing_ops = []
        for x in xs:
            if x is None:
                drawing_ops.append(_Close())
            else:
                drawing_ops.append(_BaseDrawingOperation(None, x, None))

        builder = FreeformBuilder(None, start_x, None, None, None)
        builder._drawing_operations.extend(drawing_ops)
        return builder, expected_value

    @pytest.fixture(
        params=[
            (0, (2, None, 6, 8), 0),
            (4, (2, None, 6, 8), 2),
            (19, (213, -22, None, 100), -22),
        ]
    )
    def shape_offset_y_fixture(self, request):
        start_y, ys, expected_value = request.param
        drawing_ops = []
        for y in ys:
            if y is None:
                drawing_ops.append(_Close())
            else:
                drawing_ops.append(_BaseDrawingOperation(None, None, y))

        builder = FreeformBuilder(None, None, start_y, None, None)
        builder._drawing_operations.extend(drawing_ops)
        return builder, expected_value

    @pytest.fixture
    def sp_fixture(self, _left_prop_, _top_prop_, _width_prop_, _height_prop_):
        origin_x, origin_y = 42, 24
        spTree = element("p:spTree")
        shapes = SlideShapes(spTree, None)
        _left_prop_.return_value, _top_prop_.return_value = 12, 34
        _width_prop_.return_value, _height_prop_.return_value = 56, 78

        builder = FreeformBuilder(shapes, None, None, None, None)
        expected_xml = snippet_seq("freeform")[0]
        return builder, origin_x, origin_y, spTree, expected_xml

    @pytest.fixture
    def start_path_fixture(self, _dx_prop_, _dy_prop_, _local_to_shape_):
        sp = element("p:sp/p:spPr/a:custGeom")
        start_x, start_y = 42, 24
        _dx_prop_.return_value, _dy_prop_.return_value = 1001, 2002
        _local_to_shape_.return_value = 101, 202

        builder = FreeformBuilder(None, start_x, start_y, None, None)
        expected_xml = xml(
            "p:sp/p:spPr/a:custGeom/a:pathLst/a:path{w=1001,h=2002}/a:moveTo"
            "/a:pt{x=101,y=202}"
        )
        return builder, sp, _local_to_shape_, start_x, start_y, expected_xml

    @pytest.fixture(
        params=[(0, 11.0, 0), (100, 10.36, 1036), (914242, 943.1, 862221630)]
    )
    def top_fixture(self, request, shape_offset_y_prop_):
        offset_y, y_scale, expected_value = request.param
        shape_offset_y_prop_.return_value = offset_y

        builder = FreeformBuilder(None, None, None, None, y_scale)
        return builder, expected_value

    @pytest.fixture(params=[(0, 1.0, 0), (42, 10.0, 420), (914400, 914.4, 836127360)])
    def width_fixture(self, request, _dx_prop_):
        dx, x_scale, expected_value = request.param
        _dx_prop_.return_value = dx

        builder = FreeformBuilder(None, None, None, x_scale, None)
        return builder, expected_value

    # fixture components -----------------------------------

    @pytest.fixture
    def _add_close_(self, request):
        return method_mock(request, FreeformBuilder, "_add_close")

    @pytest.fixture
    def _add_freeform_sp_(self, request):
        return method_mock(request, FreeformBuilder, "_add_freeform_sp", autospec=True)

    @pytest.fixture
    def _add_line_segment_(self, request):
        return method_mock(request, FreeformBuilder, "_add_line_segment")

    @pytest.fixture
    def apply_operation_to_(self, request):
        return method_mock(
            request, _BaseDrawingOperation, "apply_operation_to", autospec=True
        )

    @pytest.fixture
    def close_(self, request):
        return instance_mock(request, _Close)

    @pytest.fixture
    def _Close_new_(self, request):
        return method_mock(request, _Close, "new")

    @pytest.fixture
    def _dx_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_dx")

    @pytest.fixture
    def _dy_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_dy")

    @pytest.fixture
    def _height_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_height")

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, FreeformBuilder, autospec=True)

    @pytest.fixture
    def _left_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_left")

    @pytest.fixture
    def line_segment_(self, request):
        return instance_mock(request, _LineSegment)

    @pytest.fixture
    def _LineSegment_new_(self, request):
        return method_mock(request, _LineSegment, "new")

    @pytest.fixture
    def _local_to_shape_(self, request):
        return method_mock(request, FreeformBuilder, "_local_to_shape")

    @pytest.fixture
    def move_to_(self, request):
        return instance_mock(request, _MoveTo)

    @pytest.fixture
    def _MoveTo_new_(self, request):
        return method_mock(request, _MoveTo, "new")

    @pytest.fixture
    def shape_(self, request):
        return instance_mock(request, Shape)

    @pytest.fixture
    def shape_offset_x_prop_(self, request):
        return property_mock(request, FreeformBuilder, "shape_offset_x")

    @pytest.fixture
    def shape_offset_y_prop_(self, request):
        return property_mock(request, FreeformBuilder, "shape_offset_y")

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, SlideShapes)

    @pytest.fixture
    def _start_path_(self, request):
        return method_mock(request, FreeformBuilder, "_start_path", autospec=True)

    @pytest.fixture
    def _top_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_top")

    @pytest.fixture
    def _width_prop_(self, request):
        return property_mock(request, FreeformBuilder, "_width")


class Describe_BaseDrawingOperation(object):
    def it_knows_its_x_coordinate(self, x_fixture):
        drawing_operation, expected_value = x_fixture
        x = drawing_operation.x
        assert x == expected_value

    def it_knows_its_y_coordinate(self, y_fixture):
        drawing_operation, expected_value = y_fixture
        y = drawing_operation.y
        assert y == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def x_fixture(self):
        x = 42
        drawing_operation = _BaseDrawingOperation(None, x, None)
        expected_value = x
        return drawing_operation, expected_value

    @pytest.fixture
    def y_fixture(self):
        y = 24
        drawing_operation = _BaseDrawingOperation(None, None, y)
        expected_value = y
        return drawing_operation, expected_value


class Describe_Close(object):
    def it_provides_a_constructor(self, new_fixture):
        _init_ = new_fixture

        close = _Close.new()

        _init_.assert_called_once_with()
        assert isinstance(close, _Close)

    def it_can_add_close_a_contour(self, apply_fixture):
        close, path, expected_xml = apply_fixture

        close_elm = close.apply_operation_to(path)

        assert path.xml == expected_xml
        assert close_elm is path.xpath("a:close")[-1]

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def new_fixture(self, _init_):
        return _init_

    # fixture components -----------------------------------

    @pytest.fixture
    def apply_fixture(self):
        path = element("a:path")
        close = _Close()
        expected_xml = xml("a:path/a:close")
        return close, path, expected_xml

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, _Close, autospec=True)


class Describe_LineSegment(object):
    def it_provides_a_constructor(self, new_fixture):
        builder_, x, y, _init_, x_int, y_int = new_fixture

        line_segment = _LineSegment.new(builder_, x, y)

        _init_.assert_called_once_with(line_segment, builder_, x_int, y_int)
        assert isinstance(line_segment, _LineSegment)

    def it_can_add_its_line_segment_to_a_path(self, apply_fixture):
        line_segment, path, expected_xml = apply_fixture

        lnTo = line_segment.apply_operation_to(path)

        assert path.xml == expected_xml
        assert lnTo is path.xpath("a:lnTo")[-1]

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def apply_fixture(self, builder_):
        x, y = 420, 240
        path = element("a:path")
        builder_.shape_offset_x, builder_.shape_offset_y = 100, 200

        line_segment = _LineSegment(builder_, x, y)
        expected_xml = xml("a:path/a:lnTo/a:pt{x=320,y=40}")
        return line_segment, path, expected_xml

    @pytest.fixture
    def new_fixture(self, builder_, _init_):
        x, y, x_int, y_int = 99.51, 200.49, 100, 200
        return builder_, x, y, _init_, x_int, y_int

    # fixture components -----------------------------------

    @pytest.fixture
    def builder_(self, request):
        return instance_mock(request, FreeformBuilder)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, _LineSegment, autospec=True)


class Describe_MoveTo(object):
    def it_provides_a_constructor(self, new_fixture):
        builder_, x, y, _init_, x_int, y_int = new_fixture

        move_to = _MoveTo.new(builder_, x, y)

        _init_.assert_called_once_with(move_to, builder_, x_int, y_int)
        assert isinstance(move_to, _MoveTo)

    def it_can_add_its_move_to_a_path(self, apply_fixture):
        move_to, path, expected_xml = apply_fixture

        moveTo = move_to.apply_operation_to(path)

        assert path.xml == expected_xml
        assert moveTo is path.xpath("a:moveTo")[-1]

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def apply_fixture(self, builder_):
        x, y = 120, 340
        path = element("a:path")
        builder_.shape_offset_x, builder_.shape_offset_y = 100, 200

        move_to = _MoveTo(builder_, x, y)
        expected_xml = xml("a:path/a:moveTo/a:pt{x=20,y=140}")
        return move_to, path, expected_xml

    @pytest.fixture
    def new_fixture(self, builder_, _init_):
        x, y, x_int, y_int = 99.51, 200.49, 100, 200
        return builder_, x, y, _init_, x_int, y_int

    # fixture components -----------------------------------

    @pytest.fixture
    def builder_(self, request):
        return instance_mock(request, FreeformBuilder)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, _MoveTo, autospec=True)



# encoding: utf-8

"""
Base and meta classes that enable declarative definition of custom element
classes.
"""

from __future__ import absolute_import, print_function

import re

from lxml import etree

from . import oxml_parser
from ..compat import Unicode
from ..exc import InvalidXmlError
from .ns import NamespacePrefixedTag, _nsmap, qn
from ..util import lazyproperty


def OxmlElement(nsptag_str, nsmap=None):
    """
    Return a 'loose' lxml element having the tag specified by *nsptag_str*.
    *nsptag_str* must contain the standard namespace prefix, e.g. 'a:tbl'.
    The resulting element is an instance of the custom element class for this
    tag name if one is defined.
    """
    nsptag = NamespacePrefixedTag(nsptag_str)
    nsmap = nsmap if nsmap is not None else nsptag.nsmap
    return oxml_parser.makeelement(nsptag.clark_name, nsmap=nsmap)


def serialize_for_reading(element):
    """
    Serialize *element* to human-readable XML suitable for tests. No XML
    declaration.
    """
    xml = etree.tostring(element, encoding="unicode", pretty_print=True)
    return XmlString(xml)


class XmlString(Unicode):
    """
    Provides string comparison override suitable for serialized XML that is
    useful for tests.
    """

    # '    <w:xyz xmlns:a="http://ns/decl/a" attr_name="val">text</w:xyz>'
    # |          |                                          ||           |
    # +----------+------------------------------------------++-----------+
    #  front      attrs                                     | text
    #                                                     close

    _xml_elm_line_patt = re.compile(r"( *</?[\w:]+)(.*?)(/?>)([^<]*</[\w:]+>)?")

    def __eq__(self, other):
        lines = self.splitlines()
        lines_other = other.splitlines()
        if len(lines) != len(lines_other):
            return False
        for line, line_other in zip(lines, lines_other):
            if not self._eq_elm_strs(line, line_other):
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def _attr_seq(self, attrs):
        """
        Return a sequence of attribute strings parsed from *attrs*. Each
        attribute string is stripped of whitespace on both ends.
        """
        attrs = attrs.strip()
        attr_lst = attrs.split()
        return sorted(attr_lst)

    def _eq_elm_strs(self, line, line_2):
        """
        Return True if the element in *line_2* is XML equivalent to the
        element in *line*.
        """
        front, attrs, close, text = self._parse_line(line)
        front_2, attrs_2, close_2, text_2 = self._parse_line(line_2)
        if front != front_2:
            return False
        if self._attr_seq(attrs) != self._attr_seq(attrs_2):
            return False
        if close != close_2:
            return False
        if text != text_2:
            return False
        return True

    def _parse_line(self, line):
        """
        Return front, attrs, close, text 4-tuple result of parsing XML element
        string *line*.
        """
        match = self._xml_elm_line_patt.match(line)
        front, attrs, close, text = [match.group(n) for n in range(1, 5)]
        return front, attrs, close, text


class MetaOxmlElement(type):
    """
    Metaclass for BaseOxmlElement
    """

    def __init__(cls, clsname, bases, clsdict):
        dispatchable = (
            OneAndOnlyOne,
            OneOrMore,
            OptionalAttribute,
            RequiredAttribute,
            ZeroOrMore,
            ZeroOrOne,
            ZeroOrOneChoice,
        )
        for key, value in clsdict.items():
            if isinstance(value, dispatchable):
                value.populate_class_members(cls, key)


class BaseAttribute(object):
    """
    Base class for OptionalAttribute and RequiredAttribute, providing common
    methods.
    """

    def __init__(self, attr_name, simple_type):
        super(BaseAttribute, self).__init__()
        self._attr_name = attr_name
        self._simple_type = simple_type

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        self._element_cls = element_cls
        self._prop_name = prop_name

        self._add_attr_property()

    def _add_attr_property(self):
        """
        Add a read/write ``{prop_name}`` property to the element class that
        returns the interpreted value of this attribute on access and changes
        the attribute value to its ST_* counterpart on assignment.
        """
        property_ = property(self._getter, self._setter, None)
        # assign unconditionally to overwrite element name definition
        setattr(self._element_cls, self._prop_name, property_)

    @property
    def _clark_name(self):
        if ":" in self._attr_name:
            return qn(self._attr_name)
        return self._attr_name


class OptionalAttribute(BaseAttribute):
    """
    Defines an optional attribute on a custom element class. An optional
    attribute returns a default value when not present for reading. When
    assigned |None|, the attribute is removed.
    """

    def __init__(self, attr_name, simple_type, default=None):
        super(OptionalAttribute, self).__init__(attr_name, simple_type)
        self._default = default

    @property
    def _docstring(self):
        """
        Return the string to use as the ``__doc__`` attribute of the property
        for this attribute.
        """
        return (
            "%s type-converted value of ``%s`` attribute, or |None| (or spec"
            "ified default value) if not present. Assigning the default valu"
            "e causes the attribute to be removed from the element."
            % (self._simple_type.__name__, self._attr_name)
        )

    @property
    def _getter(self):
        """
        Return a function object suitable for the "get" side of the attribute
        property descriptor.
        """

        def get_attr_value(obj):
            attr_str_value = obj.get(self._clark_name)
            if attr_str_value is None:
                return self._default
            return self._simple_type.from_xml(attr_str_value)

        get_attr_value.__doc__ = self._docstring
        return get_attr_value

    @property
    def _setter(self):
        """
        Return a function object suitable for the "set" side of the attribute
        property descriptor.
        """

        def set_attr_value(obj, value):
            if value == self._default:
                if self._clark_name in obj.attrib:
                    del obj.attrib[self._clark_name]
                return
            str_value = self._simple_type.to_xml(value)
            obj.set(self._clark_name, str_value)

        return set_attr_value


class RequiredAttribute(BaseAttribute):
    """
    Defines a required attribute on a custom element class. A required
    attribute is assumed to be present for reading, so does not have
    a default value; its actual value is always used. If missing on read,
    an |InvalidXmlError| is raised. It also does not remove the attribute if
    |None| is assigned. Assigning |None| raises |TypeError| or |ValueError|,
    depending on the simple type of the attribute.
    """

    @property
    def _getter(self):
        """
        Return a function object suitable for the "get" side of the attribute
        property descriptor.
        """

        def get_attr_value(obj):
            attr_str_value = obj.get(self._clark_name)
            if attr_str_value is None:
                raise InvalidXmlError(
                    "required '%s' attribute not present on element %s"
                    % (self._attr_name, obj.tag)
                )
            return self._simple_type.from_xml(attr_str_value)

        get_attr_value.__doc__ = self._docstring
        return get_attr_value

    @property
    def _docstring(self):
        """
        Return the string to use as the ``__doc__`` attribute of the property
        for this attribute.
        """
        return "%s type-converted value of ``%s`` attribute." % (
            self._simple_type.__name__,
            self._attr_name,
        )

    @property
    def _setter(self):
        """
        Return a function object suitable for the "set" side of the attribute
        property descriptor.
        """

        def set_attr_value(obj, value):
            str_value = self._simple_type.to_xml(value)
            obj.set(self._clark_name, str_value)

        return set_attr_value


class _BaseChildElement(object):
    """
    Base class for the child element classes corresponding to varying
    cardinalities, such as ZeroOrOne and ZeroOrMore.
    """

    def __init__(self, nsptagname, successors=()):
        super(_BaseChildElement, self).__init__()
        self._nsptagname = nsptagname
        self._successors = successors

    def populate_class_members(self, element_cls, prop_name):
        """
        Baseline behavior for adding the appropriate methods to
        *element_cls*.
        """
        self._element_cls = element_cls
        self._prop_name = prop_name

    def _add_adder(self):
        """
        Add an ``_add_x()`` method to the element class for this child
        element.
        """

        def _add_child(obj, **attrs):
            new_method = getattr(obj, self._new_method_name)
            child = new_method()
            for key, value in attrs.items():
                setattr(child, key, value)
            insert_method = getattr(obj, self._insert_method_name)
            insert_method(child)
            return child

        _add_child.__doc__ = (
            "Add a new ``<%s>`` child element unconditionally, inserted in t"
            "he correct sequence." % self._nsptagname
        )
        self._add_to_class(self._add_method_name, _add_child)

    def _add_creator(self):
        """
        Add a ``_new_{prop_name}()`` method to the element class that creates
        a new, empty element of the correct type, having no attributes.
        """
        creator = self._creator
        creator.__doc__ = (
            'Return a "loose", newly created ``<%s>`` element having no attri'
            "butes, text, or children." % self._nsptagname
        )
        self._add_to_class(self._new_method_name, creator)

    def _add_getter(self):
        """
        Add a read-only ``{prop_name}`` property to the element class for
        this child element.
        """
        property_ = property(self._getter, None, None)
        # assign unconditionally to overwrite element name definition
        setattr(self._element_cls, self._prop_name, property_)

    def _add_inserter(self):
        """
        Add an ``_insert_x()`` method to the element class for this child
        element.
        """

        def _insert_child(obj, child):
            obj.insert_element_before(child, *self._successors)
            return child

        _insert_child.__doc__ = (
            "Return the passed ``<%s>`` element after inserting it as a chil"
            "d in the correct sequence." % self._nsptagname
        )
        self._add_to_class(self._insert_method_name, _insert_child)

    def _add_list_getter(self):
        """
        Add a read-only ``{prop_name}_lst`` property to the element class to
        retrieve a list of child elements matching this type.
        """
        prop_name = "%s_lst" % self._prop_name
        property_ = property(self._list_getter, None, None)
        setattr(self._element_cls, prop_name, property_)

    @lazyproperty
    def _add_method_name(self):
        return "_add_%s" % self._prop_name

    def _add_to_class(self, name, method):
        """
        Add *method* to the target class as *name*, unless *name* is already
        defined on the class.
        """
        if hasattr(self._element_cls, name):
            return
        setattr(self._element_cls, name, method)

    @property
    def _creator(self):
        """
        Return a function object that creates a new, empty element of the
        right type, having no attributes.
        """

        def new_child_element(obj):
            return OxmlElement(self._nsptagname)

        return new_child_element

    @property
    def _getter(self):
        """
        Return a function object suitable for the "get" side of the property
        descriptor. This default getter returns the child element with
        matching tag name or |None| if not present.
        """

        def get_child_element(obj):
            return obj.find(qn(self._nsptagname))

        get_child_element.__doc__ = (
            "``<%s>`` child element or |None| if not present." % self._nsptagname
        )
        return get_child_element

    @lazyproperty
    def _insert_method_name(self):
        return "_insert_%s" % self._prop_name

    @property
    def _list_getter(self):
        """
        Return a function object suitable for the "get" side of a list
        property descriptor.
        """

        def get_child_element_list(obj):
            return obj.findall(qn(self._nsptagname))

        get_child_element_list.__doc__ = (
            "A list containing each of the ``<%s>`` child elements, in the o"
            "rder they appear." % self._nsptagname
        )
        return get_child_element_list

    @lazyproperty
    def _remove_method_name(self):
        return "_remove_%s" % self._prop_name

    @lazyproperty
    def _new_method_name(self):
        return "_new_%s" % self._prop_name


class Choice(_BaseChildElement):
    """
    Defines a child element belonging to a group, only one of which may
    appear as a child.
    """

    @property
    def nsptagname(self):
        return self._nsptagname

    def populate_class_members(self, element_cls, group_prop_name, successors):
        """
        Add the appropriate methods to *element_cls*.
        """
        self._element_cls = element_cls
        self._group_prop_name = group_prop_name
        self._successors = successors

        self._add_getter()
        self._add_creator()
        self._add_inserter()
        self._add_adder()
        self._add_get_or_change_to_method()

    def _add_get_or_change_to_method(self):
        """
        Add a ``get_or_change_to_x()`` method to the element class for this
        child element.
        """

        def get_or_change_to_child(obj):
            child = getattr(obj, self._prop_name)
            if child is not None:
                return child
            remove_group_method = getattr(obj, self._remove_group_method_name)
            remove_group_method()
            add_method = getattr(obj, self._add_method_name)
            child = add_method()
            return child

        get_or_change_to_child.__doc__ = (
            "Return the ``<%s>`` child, replacing any other group element if" " found."
        ) % self._nsptagname
        self._add_to_class(self._get_or_change_to_method_name, get_or_change_to_child)

    @property
    def _prop_name(self):
        """
        Calculate property name from tag name, e.g. a:schemeClr -> schemeClr.
        """
        if ":" in self._nsptagname:
            start = self._nsptagname.index(":") + 1
        else:
            start = 0
        return self._nsptagname[start:]

    @lazyproperty
    def _get_or_change_to_method_name(self):
        return "get_or_change_to_%s" % self._prop_name

    @lazyproperty
    def _remove_group_method_name(self):
        return "_remove_%s" % self._group_prop_name


class OneAndOnlyOne(_BaseChildElement):
    """
    Defines a required child element for MetaOxmlElement.
    """

    def __init__(self, nsptagname):
        super(OneAndOnlyOne, self).__init__(nsptagname, None)

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        super(OneAndOnlyOne, self).populate_class_members(element_cls, prop_name)
        self._add_getter()

    @property
    def _getter(self):
        """
        Return a function object suitable for the "get" side of the property
        descriptor.
        """

        def get_child_element(obj):
            child = obj.find(qn(self._nsptagname))
            if child is None:
                raise InvalidXmlError(
                    "required ``<%s>`` child element not present" % self._nsptagname
                )
            return child

        get_child_element.__doc__ = (
            "Required ``<%s>`` child element." % self._nsptagname
        )
        return get_child_element


class OneOrMore(_BaseChildElement):
    """
    Defines a repeating child element for MetaOxmlElement that must appear at
    least once.
    """

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        super(OneOrMore, self).populate_class_members(element_cls, prop_name)
        self._add_list_getter()
        self._add_creator()
        self._add_inserter()
        self._add_adder()
        self._add_public_adder()
        delattr(element_cls, prop_name)

    def _add_public_adder(self):
        """
        Add a public ``add_x()`` method to the parent element class.
        """

        def add_child(obj):
            private_add_method = getattr(obj, self._add_method_name)
            child = private_add_method()
            return child

        add_child.__doc__ = (
            "Add a new ``<%s>`` child element unconditionally, inserted in t"
            "he correct sequence." % self._nsptagname
        )
        self._add_to_class(self._public_add_method_name, add_child)

    @lazyproperty
    def _public_add_method_name(self):
        """
        add_childElement() is public API for a repeating element, allowing
        new elements to be added to the sequence. May be overridden to
        provide a friendlier API to clients having domain appropriate
        parameter names for required attributes.
        """
        return "add_%s" % self._prop_name


class ZeroOrMore(_BaseChildElement):
    """
    Defines an optional repeating child element for MetaOxmlElement.
    """

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        super(ZeroOrMore, self).populate_class_members(element_cls, prop_name)
        self._add_list_getter()
        self._add_creator()
        self._add_inserter()
        self._add_adder()
        delattr(element_cls, prop_name)


class ZeroOrOne(_BaseChildElement):
    """
    Defines an optional child element for MetaOxmlElement.
    """

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        super(ZeroOrOne, self).populate_class_members(element_cls, prop_name)
        self._add_getter()
        self._add_creator()
        self._add_inserter()
        self._add_adder()
        self._add_get_or_adder()
        self._add_remover()

    def _add_get_or_adder(self):
        """
        Add a ``get_or_add_x()`` method to the element class for this
        child element.
        """

        def get_or_add_child(obj):
            child = getattr(obj, self._prop_name)
            if child is None:
                add_method = getattr(obj, self._add_method_name)
                child = add_method()
            return child

        get_or_add_child.__doc__ = (
            "Return the ``<%s>`` child element, newly added if not present."
        ) % self._nsptagname
        self._add_to_class(self._get_or_add_method_name, get_or_add_child)

    def _add_remover(self):
        """
        Add a ``_remove_x()`` method to the element class for this child
        element.
        """

        def _remove_child(obj):
            obj.remove_all(self._nsptagname)

        _remove_child.__doc__ = (
            "Remove all ``<%s>`` child elements."
        ) % self._nsptagname
        self._add_to_class(self._remove_method_name, _remove_child)

    @lazyproperty
    def _get_or_add_method_name(self):
        return "get_or_add_%s" % self._prop_name


class ZeroOrOneChoice(_BaseChildElement):
    """
    Correspondes to an ``EG_*`` element group where at most one of its
    members may appear as a child.
    """

    def __init__(self, choices, successors=()):
        self._choices = choices
        self._successors = successors

    def populate_class_members(self, element_cls, prop_name):
        """
        Add the appropriate methods to *element_cls*.
        """
        super(ZeroOrOneChoice, self).populate_class_members(element_cls, prop_name)
        self._add_choice_getter()
        for choice in self._choices:
            choice.populate_class_members(
                element_cls, self._prop_name, self._successors
            )
        self._add_group_remover()

    def _add_choice_getter(self):
        """
        Add a read-only ``{prop_name}`` property to the element class that
        returns the present member of this group, or |None| if none are
        present.
        """
        property_ = property(self._choice_getter, None, None)
        # assign unconditionally to overwrite element name definition
        setattr(self._element_cls, self._prop_name, property_)

    def _add_group_remover(self):
        """
        Add a ``_remove_eg_x()`` method to the element class for this choice
        group.
        """

        def _remove_choice_group(obj):
            for tagname in self._member_nsptagnames:
                obj.remove_all(tagname)

        _remove_choice_group.__doc__ = (
            "Remove the current choice group child element if present."
        )
        self._add_to_class(self._remove_choice_group_method_name, _remove_choice_group)

    @property
    def _choice_getter(self):
        """
        Return a function object suitable for the "get" side of the property
        descriptor.
        """

        def get_group_member_element(obj):
            return obj.first_child_found_in(*self._member_nsptagnames)

        get_group_member_element.__doc__ = (
            "Return the child element belonging to this element group, or "
            "|None| if no member child is present."
        )
        return get_group_member_element

    @lazyproperty
    def _member_nsptagnames(self):
        """
        Sequence of namespace-prefixed tagnames, one for each of the member
        elements of this choice group.
        """
        return [choice.nsptagname for choice in self._choices]

    @lazyproperty
    def _remove_choice_group_method_name(self):
        return "_remove_%s" % self._prop_name


class _OxmlElementBase(etree.ElementBase):
    """
    Provides common behavior for oxml element classes
    """

    @classmethod
    def child_tagnames_after(cls, tagname):
        """
        Return a sequence containing the namespace prefixed child tagnames,
        e.g. 'a:prstGeom', that occur after *tagname* in this element.
        """
        return cls.child_tagnames.tagnames_after(tagname)

    def delete(self):
        """
        Remove this element from the XML tree.
        """
        self.getparent().remove(self)

    def first_child_found_in(self, *tagnames):
        """
        Return the first child found with tag in *tagnames*, or None if
        not found.
        """
        for tagname in tagnames:
            child = self.find(qn(tagname))
            if child is not None:
                return child
        return None

    def insert_element_before(self, elm, *tagnames):
        successor = self.first_child_found_in(*tagnames)
        if successor is not None:
            successor.addprevious(elm)
        else:
            self.append(elm)
        return elm

    def remove_all(self, tagname):
        """
        Remove all child elements having *tagname*.
        """
        matching = self.findall(qn(tagname))
        for child in matching:
            self.remove(child)

    def remove_if_present(self, *tagnames):
        """
        Remove all child elements having tagname in *tagnames*.
        """
        for tagname in tagnames:
            element = self.find(qn(tagname))
            if element is not None:
                self.remove(element)

    @property
    def xml(self):
        """
        Return XML string for this element, suitable for testing purposes.
        Pretty printed for readability and without an XML declaration at the
        top.
        """
        return serialize_for_reading(self)

    def xpath(self, xpath_str):
        """
        Override of ``lxml`` _Element.xpath() method to provide standard Open
        XML namespace mapping in centralized location.
        """
        return super(BaseOxmlElement, self).xpath(xpath_str, namespaces=_nsmap)


BaseOxmlElement = MetaOxmlElement(
    "BaseOxmlElement", (etree.ElementBase,), dict(_OxmlElementBase.__dict__)
)



# encoding: utf-8

"""Unit-test suite for pptx.oxml.table module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.oxml.ns import nsdecls
from pptx.oxml.table import CT_Table, TcRange

from ..unitutil.cxml import element


class DescribeCT_Table(object):
    def it_can_create_a_new_tbl_element_tree(self):
        """
        Indirectly tests that column widths are a proportional split of total
        width and that row heights a proportional split of total height.
        """
        expected_xml = (
            '<a:tbl %s>\n  <a:tblPr firstRow="1" bandRow="1">\n    <a:tableSt'
            "yleId>{5C22544A-7EE6-4342-B048-85BDC9FD1C3A}</a:tableStyleId>\n "
            ' </a:tblPr>\n  <a:tblGrid>\n    <a:gridCol w="111"/>\n    <a:gri'
            'dCol w="111"/>\n    <a:gridCol w="112"/>\n  </a:tblGrid>\n  <a:t'
            'r h="222">\n    <a:tc>\n      <a:txBody>\n        <a:bodyPr/>\n '
            "       <a:lstStyle/>\n        <a:p/>\n      </a:txBody>\n      <"
            "a:tcPr/>\n    </a:tc>\n    <a:tc>\n      <a:txBody>\n        <a:"
            "bodyPr/>\n        <a:lstStyle/>\n        <a:p/>\n      </a:txBod"
            "y>\n      <a:tcPr/>\n    </a:tc>\n    <a:tc>\n      <a:txBody>\n"
            "        <a:bodyPr/>\n        <a:lstStyle/>\n        <a:p/>\n    "
            "  </a:txBody>\n      <a:tcPr/>\n    </a:tc>\n  </a:tr>\n  <a:tr "
            'h="223">\n    <a:tc>\n      <a:txBody>\n        <a:bodyPr/>\n   '
            "     <a:lstStyle/>\n        <a:p/>\n      </a:txBody>\n      <a:"
            "tcPr/>\n    </a:tc>\n    <a:tc>\n      <a:txBody>\n        <a:bo"
            "dyPr/>\n        <a:lstStyle/>\n        <a:p/>\n      </a:txBody>"
            "\n      <a:tcPr/>\n    </a:tc>\n    <a:tc>\n      <a:txBody>\n  "
            "      <a:bodyPr/>\n        <a:lstStyle/>\n        <a:p/>\n      "
            "</a:txBody>\n      <a:tcPr/>\n    </a:tc>\n  </a:tr>\n</a:tbl>\n"
            % nsdecls("a")
        )
        tbl = CT_Table.new_tbl(2, 3, 334, 445)
        assert tbl.xml == expected_xml

    def it_provides_access_to_its_tc_elements(self):
        tbl_cxml = "a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))"
        tbl = element(tbl_cxml)
        tcs = tbl.xpath(".//a:tc")

        assert tbl.tc(0, 0) is tcs[0]
        assert tbl.tc(0, 1) is tcs[1]
        assert tbl.tc(1, 0) is tcs[2]
        assert tbl.tc(1, 1) is tcs[3]


class DescribeTcRange(object):
    def it_knows_when_the_range_contains_a_merged_cell(self, contains_merge_fixture):
        tc, other_tc, expected_value = contains_merge_fixture
        tc_range = TcRange(tc, other_tc)

        contains_merged_cell = tc_range.contains_merged_cell

        assert contains_merged_cell is expected_value

    def it_knows_how_big_the_merge_range_is(self, dimensions_fixture):
        tc, other_tc, expected_value = dimensions_fixture
        tc_range = TcRange(tc, other_tc)

        dimensions = tc_range.dimensions

        assert dimensions == expected_value

    def it_knows_when_tcs_are_in_the_same_tbl(self, in_same_table_fixture):
        tc, other_tc, expected_value = in_same_table_fixture
        tc_range = TcRange(tc, other_tc)

        in_same_table = tc_range.in_same_table

        assert in_same_table is expected_value

    def it_can_iterate_tcs_not_in_left_col_of_range(self, except_left_fixture):
        tc, other_tc, expected_value = except_left_fixture
        tc_range = TcRange(tc, other_tc)

        tcs = list(tc_range.iter_except_left_col_tcs())

        assert tcs == expected_value

    def it_can_iterate_tcs_not_in_top_row_of_range(self, except_top_fixture):
        tc, other_tc, expected_value = except_top_fixture
        tc_range = TcRange(tc, other_tc)

        tcs = list(tc_range.iter_except_top_row_tcs())

        assert tcs == expected_value

    def it_can_iterate_left_col_of_range_tcs(self, left_col_fixture):
        tc, other_tc, expected_value = left_col_fixture
        tc_range = TcRange(tc, other_tc)

        tcs = list(tc_range.iter_left_col_tcs())

        assert tcs == expected_value

    def it_can_iterate_top_row_of_range_tcs(self, top_row_fixture):
        tc, other_tc, expected_value = top_row_fixture
        tc_range = TcRange(tc, other_tc)

        tcs = list(tc_range.iter_top_row_tcs())

        assert tcs == expected_value

    def it_can_migrate_range_content_to_origin_cell(self, move_fixture):
        tc, other_tc, expected_text = move_fixture
        tc_range = TcRange(tc, other_tc)

        tc_range.move_content_to_origin()

        assert tc.text == expected_text
        assert other_tc.text == ""

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", False),
            ("a:tbl/a:tr/(a:tc{gridSpan=1},a:tc{hMerge=false})", False),
            ("a:tbl/a:tr/(a:tc{gridSpan=2},a:tc{hMerge=1})", True),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", False),
            ("a:tbl/(a:tr/a:tc{rowSpan=1},a:tr/a:tc{vMerge=false})", False),
            ("a:tbl/(a:tr/a:tc{rowSpan=2},a:tr/a:tc{vMerge=true})", True),
        ]
    )
    def contains_merge_fixture(self, request):
        tbl_cxml, expected_value = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        return tcs[0], tcs[1], expected_value

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", (1, 2)),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", (2, 1)),
            ("a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))", (2, 2)),
        ]
    )
    def dimensions_fixture(self, request):
        tbl_cxml, expected_value = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        return tcs[0], tcs[-1], expected_value

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", [0, 1], [1]),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", [0, 1], []),
            ("a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))", [2, 1], [1, 3]),
            (
                "a:tbl/(a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc"
                ",a:tc))",
                [0, 8],
                [1, 2, 4, 5, 7, 8],
            ),
        ]
    )
    def except_left_fixture(self, request):
        tbl_cxml, tc_idxs, expected_tc_idxs = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        tc, other_tc = tcs[tc_idxs[0]], tcs[tc_idxs[1]]
        expected_value = [tcs[idx] for idx in expected_tc_idxs]
        return tc, other_tc, expected_value

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", [0, 1], []),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", [0, 1], [1]),
            ("a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))", [2, 1], [2, 3]),
            (
                "a:tbl/(a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc"
                ",a:tc))",
                [0, 8],
                [3, 4, 5, 6, 7, 8],
            ),
        ]
    )
    def except_top_fixture(self, request):
        tbl_cxml, tc_idxs, expected_tc_idxs = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        tc, other_tc = tcs[tc_idxs[0]], tcs[tc_idxs[1]]
        expected_value = [tcs[idx] for idx in expected_tc_idxs]
        return tc, other_tc, expected_value

    @pytest.fixture(params=[True, False])
    def in_same_table_fixture(self, request):
        expected_value = request.param
        tbl = element("a:tbl/a:tr/(a:tc,a:tc)")
        other_tbl = element("a:tbl/a:tr/(a:tc,a:tc)")
        tc = tbl.xpath("//a:tc")[0]
        other_tc = (
            tbl.xpath("//a:tc")[1] if expected_value else other_tbl.xpath("//a:tc")[1]
        )
        return tc, other_tc, expected_value

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", (0, 1), (0,)),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", (0, 1), (0, 1)),
            ("a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))", (2, 1), (0, 2)),
            (
                "a:tbl/(a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc"
                ",a:tc))",
                (4, 8),
                (4, 7),
            ),
        ]
    )
    def left_col_fixture(self, request):
        tbl_cxml, tc_idxs, expected_tc_idxs = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        tc, other_tc = tcs[tc_idxs[0]], tcs[tc_idxs[1]]
        expected_value = [tcs[idx] for idx in expected_tc_idxs]
        return tc, other_tc, expected_value

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc/a:txBody/a:p,a:tc/a:txBody/a:p)", ""),
            ('a:tbl/a:tr/(a:tc/a:txBody/a:p,a:tc/a:txBody/a:p/a:r/a:t"b")', "b"),
            ('a:tbl/a:tr/(a:tc/a:txBody/a:p/a:r/a:t"a",a:tc/a:txBody/a:p)', "a"),
            (
                'a:tbl/a:tr/(a:tc/a:txBody/a:p/a:r/a:t"a",a:tc/a:txBody/a:p/a:r/a:t'
                '"b")',
                "a\nb",
            ),
            (
                'a:tbl/a:tr/(a:tc/a:txBody/a:p/a:r/a:t"a",a:tc/a:txBody/(a:p,a:p))',
                "a\n\n",
            ),
            (
                'a:tbl/a:tr/(a:tc/a:txBody/(a:p,a:p),a:tc/a:txBody/a:p/a:r/a:t"b")',
                "\n\nb",
            ),
        ]
    )
    def move_fixture(self, request):
        tbl_cxml, expected_text = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        return tcs[0], tcs[1], expected_text

    @pytest.fixture(
        params=[
            ("a:tbl/a:tr/(a:tc,a:tc)", (0, 1), (0, 1)),
            ("a:tbl/(a:tr/a:tc,a:tr/a:tc)", (0, 1), (0,)),
            ("a:tbl/(a:tr/(a:tc,a:tc),a:tr/(a:tc,a:tc))", (2, 1), (0, 1)),
            (
                "a:tbl/(a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc,a:tc),a:tr/(a:tc,a:tc"
                ",a:tc))",
                (4, 8),
                (4, 5),
            ),
        ]
    )
    def top_row_fixture(self, request):
        tbl_cxml, tc_idxs, expected_tc_idxs = request.param
        tcs = element(tbl_cxml).xpath("//a:tc")
        tc, other_tc = tcs[tc_idxs[0]], tcs[tc_idxs[1]]
        expected_value = [tcs[idx] for idx in expected_tc_idxs]
        return tc, other_tc, expected_value



# encoding: utf-8

"""Unit-test suite for pptx.shapes.graphfrm module."""

import pytest

from pptx.chart.chart import Chart
from pptx.enum.shapes import MSO_SHAPE_TYPE
from pptx.parts.chart import ChartPart
from pptx.parts.embeddedpackage import EmbeddedPackagePart
from pptx.parts.slide import SlidePart
from pptx.shapes.graphfrm import GraphicFrame, _OleFormat
from pptx.shapes.shapetree import SlideShapes
from pptx.spec import (
    GRAPHIC_DATA_URI_CHART,
    GRAPHIC_DATA_URI_OLEOBJ,
    GRAPHIC_DATA_URI_TABLE,
)

from ..unitutil.cxml import element
from ..unitutil.mock import class_mock, instance_mock, property_mock


class DescribeGraphicFrame(object):
    """Unit-test suite for `pptx.shapes.graphfrm.GraphicFrame` object."""

    def it_provides_access_to_the_chart_it_contains(
        self, request, has_chart_prop_, chart_part_, chart_
    ):
        has_chart_prop_.return_value = True
        property_mock(request, GraphicFrame, "chart_part", return_value=chart_part_)
        chart_part_.chart = chart_

        assert GraphicFrame(None, None).chart is chart_

    def but_it_raises_on_chart_if_there_isnt_one(self, has_chart_prop_):
        has_chart_prop_.return_value = False

        with pytest.raises(ValueError) as e:
            GraphicFrame(None, None).chart
        assert str(e.value) == "shape does not contain a chart"

    def it_provides_access_to_its_chart_part(self, request, chart_part_):
        graphicFrame = element(
            "p:graphicFrame/a:graphic/a:graphicData/c:chart{r:id=rId42}"
        )
        property_mock(
            request,
            GraphicFrame,
            "part",
            return_value=instance_mock(
                request, SlidePart, related_parts={"rId42": chart_part_}
            ),
        )
        graphic_frame = GraphicFrame(graphicFrame, None)

        assert graphic_frame.chart_part is chart_part_

    @pytest.mark.parametrize(
        "graphicData_uri, expected_value",
        (
            (GRAPHIC_DATA_URI_CHART, True),
            (GRAPHIC_DATA_URI_OLEOBJ, False),
            (GRAPHIC_DATA_URI_TABLE, False),
        ),
    )
    def it_knows_whether_it_contains_a_chart(self, graphicData_uri, expected_value):
        graphicFrame = element(
            "p:graphicFrame/a:graphic/a:graphicData{uri=%s}" % graphicData_uri
        )
        assert GraphicFrame(graphicFrame, None).has_chart is expected_value

    @pytest.mark.parametrize(
        "graphicData_uri, expected_value",
        (
            (GRAPHIC_DATA_URI_CHART, False),
            (GRAPHIC_DATA_URI_OLEOBJ, False),
            (GRAPHIC_DATA_URI_TABLE, True),
        ),
    )
    def it_knows_whether_it_contains_a_table(self, graphicData_uri, expected_value):
        graphicFrame = element(
            "p:graphicFrame/a:graphic/a:graphicData{uri=%s}" % graphicData_uri
        )
        assert GraphicFrame(graphicFrame, None).has_table is expected_value

    def it_provides_access_to_the_OleFormat_object(self, request):
        ole_format_ = instance_mock(request, _OleFormat)
        _OleFormat_ = class_mock(
            request, "pptx.shapes.graphfrm._OleFormat", return_value=ole_format_
        )
        graphicFrame = element(
            "p:graphicFrame/a:graphic/a:graphicData{uri=http://schemas.openxmlformats"
            ".org/presentationml/2006/ole}"
        )
        parent_ = instance_mock(request, SlideShapes)
        graphic_frame = GraphicFrame(graphicFrame, parent_)

        ole_format = graphic_frame.ole_format

        _OleFormat_.assert_called_once_with(graphicFrame.graphicData, parent_)
        assert ole_format is ole_format_

    def but_it_raises_on_ole_format_when_this_is_not_an_OLE_object(self):
        graphic_frame = GraphicFrame(
            element(
                "p:graphicFrame/a:graphic/a:graphicData{uri=http://schemas.openxmlfor"
                "mats.org/drawingml/2006/table}"
            ),
            None,
        )
        with pytest.raises(ValueError) as e:
            graphic_frame.ole_format
        assert str(e.value) == "not an OLE-object shape"

    def it_raises_on_shadow(self):
        graphic_frame = GraphicFrame(None, None)
        with pytest.raises(NotImplementedError):
            graphic_frame.shadow

    @pytest.mark.parametrize(
        "uri, oleObj_child, expected_value",
        (
            (GRAPHIC_DATA_URI_CHART, None, MSO_SHAPE_TYPE.CHART),
            (GRAPHIC_DATA_URI_OLEOBJ, "embed", MSO_SHAPE_TYPE.EMBEDDED_OLE_OBJECT),
            (GRAPHIC_DATA_URI_OLEOBJ, "link", MSO_SHAPE_TYPE.LINKED_OLE_OBJECT),
            (GRAPHIC_DATA_URI_TABLE, None, MSO_SHAPE_TYPE.TABLE),
            ("foobar", None, None),
        ),
    )
    def it_knows_its_shape_type(self, uri, oleObj_child, expected_value):
        graphicFrame = element(
            (
                "p:graphicFrame/a:graphic/a:graphicData{uri=%s}/p:oleObj/p:%s"
                % (uri, oleObj_child)
            )
            if oleObj_child
            else "p:graphicFrame/a:graphic/a:graphicData{uri=%s}" % uri
        )
        assert GraphicFrame(graphicFrame, None).shape_type is expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def chart_(self, request):
        return instance_mock(request, Chart)

    @pytest.fixture
    def chart_part_(self, request, chart_):
        return instance_mock(request, ChartPart, chart=chart_)

    @pytest.fixture
    def has_chart_prop_(self, request):
        return property_mock(request, GraphicFrame, "has_chart")


class Describe_OleFormat(object):
    """Unit-test suite for `pptx.shapes.graphfrm._OleFormat` object."""

    def it_provides_access_to_the_OLE_object_blob(self, request):
        ole_obj_part_ = instance_mock(request, EmbeddedPackagePart, blob=b"0123456789")
        property_mock(
            request,
            _OleFormat,
            "part",
            return_value=instance_mock(
                request, SlidePart, related_parts={"rId7": ole_obj_part_}
            ),
        )
        graphicData = element("a:graphicData/p:oleObj{r:id=rId7}")

        assert _OleFormat(graphicData, None).blob == b"0123456789"

    def it_knows_the_OLE_object_prog_id(self):
        graphicData = element("a:graphicData/p:oleObj{progId=Excel.Sheet.12}")
        assert _OleFormat(graphicData, None).prog_id == "Excel.Sheet.12"

    def it_knows_whether_to_show_the_OLE_object_as_an_icon(self):
        graphicData = element("a:graphicData/p:oleObj{showAsIcon=1}")
        assert _OleFormat(graphicData, None).show_as_icon is True



# encoding: utf-8

"""
Provides mappings that embody aspects of the Open XML spec ISO/IEC 29500.
"""

from .constants import CONTENT_TYPE as CT


default_content_types = (
    ("bin", CT.PML_PRINTER_SETTINGS),
    ("bin", CT.SML_PRINTER_SETTINGS),
    ("bin", CT.WML_PRINTER_SETTINGS),
    ("bmp", CT.BMP),
    ("emf", CT.X_EMF),
    ("fntdata", CT.X_FONTDATA),
    ("gif", CT.GIF),
    ("jpe", CT.JPEG),
    ("jpeg", CT.JPEG),
    ("jpg", CT.JPEG),
    ("mov", CT.MOV),
    ("mp4", CT.MP4),
    ("mpg", CT.MPG),
    ("png", CT.PNG),
    ("rels", CT.OPC_RELATIONSHIPS),
    ("tif", CT.TIFF),
    ("tiff", CT.TIFF),
    ("vid", CT.VIDEO),
    ("wdp", CT.MS_PHOTO),
    ("wmf", CT.X_WMF),
    ("wmv", CT.WMV),
    ("xlsx", CT.SML_SHEET),
    ("xml", CT.XML),
)


image_content_types = {
    "bmp": CT.BMP,
    "emf": CT.X_EMF,
    "gif": CT.GIF,
    "jpe": CT.JPEG,
    "jpeg": CT.JPEG,
    "jpg": CT.JPEG,
    "png": CT.PNG,
    "tif": CT.TIFF,
    "tiff": CT.TIFF,
    "wdp": CT.MS_PHOTO,
    "wmf": CT.X_WMF,
}



# encoding: utf-8

"""Step implementations for LineFormat-related features."""

from __future__ import absolute_import, print_function, unicode_literals

from behave import given, then, when

from pptx import Presentation
from pptx.enum.dml import MSO_LINE
from pptx.util import Length, Pt

from helpers import test_pptx


# given ===================================================


@given("a LineFormat object as line")
def given_a_LineFormat_object_as_line(context):
    line = Presentation(test_pptx("dml-line")).slides[0].shapes[0].line
    context.line = line


@given("a LineFormat object as line having {current} dash style")
def given_a_LineFormat_object_as_line_having_dash_style(context, current):
    shape_idx = {"no explicit": 0, "solid": 1, "dashed": 2, "dash-dot": 3}[current]
    shape = Presentation(test_pptx("dml-line")).slides[3].shapes[shape_idx]
    context.line = shape.line


@given("a LineFormat object as line having {line_width} width")
def given_a_LineFormat_object_as_line_having_width(context, line_width):
    shape_idx = {"no explicit": 0, "1 pt": 1}[line_width]
    prs = Presentation(test_pptx("dml-line"))
    shape = prs.slides[2].shapes[shape_idx]
    context.line = shape.line


# when ====================================================


@when("I assign {value_key} to line.dash_style")
def when_I_assign_value_to_line_dash_style(context, value_key):
    value = {
        "None": None,
        "MSO_LINE.DASH": MSO_LINE.DASH,
        "MSO_LINE.DASH_DOT": MSO_LINE.DASH_DOT,
        "MSO_LINE.SOLID": MSO_LINE.SOLID,
    }[value_key]
    context.line.dash_style = value


@when("I assign {line_width} to line.width")
def when_I_assign_value_to_line_width(context, line_width):
    value = {"None": None, "1 pt": Pt(1), "2.34 pt": Pt(2.34)}[line_width]
    context.line.width = value


# then ====================================================


@then("line.color is a ColorFormat object")
def then_line_color_is_a_ColorFormat_object(context):
    class_name = context.line.color.__class__.__name__
    expected_value = "ColorFormat"
    assert class_name == expected_value, "expected '%s', got '%s'" % (
        expected_value,
        class_name,
    )


@then("line.dash_style is {dash_style}")
def then_line_dash_style_is_value(context, dash_style):
    expected_value = {
        "None": None,
        "MSO_LINE.DASH": MSO_LINE.DASH,
        "MSO_LINE.DASH_DOT": MSO_LINE.DASH_DOT,
        "MSO_LINE.SOLID": MSO_LINE.SOLID,
    }[dash_style]
    actual_value = context.line.dash_style
    assert actual_value == expected_value, "expected %s, got %s" % (
        expected_value,
        actual_value,
    )


@then("line.fill is a FillFormat object")
def then_line_fill_is_a_FillFormat_object(context):
    class_name = context.line.fill.__class__.__name__
    expected_value = "FillFormat"
    assert class_name == expected_value, "expected '%s', got '%s'" % (
        expected_value,
        class_name,
    )


@then("line.width is {line_width}")
def then_line_width_is_value(context, line_width):
    expected_value = {"0": 0, "1 pt": Pt(1), "2.34 pt": Pt(2.34)}[line_width]
    line_width = context.line.width
    assert line_width == expected_value
    assert isinstance(line_width, Length)



# encoding: utf-8

"""
Unit test suite for the pptx.chart.point module.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.chart.datalabel import DataLabel
from pptx.chart.marker import Marker
from pptx.chart.point import BubblePoints, CategoryPoints, Point, XyPoints
from pptx.dml.chtfmt import ChartFormat

from ..unitutil.cxml import element, xml
from ..unitutil.mock import class_mock, instance_mock


class Describe_BasePoints(object):
    def it_supports_indexed_access(self, getitem_fixture):
        points, idx, Point_, ser, point_ = getitem_fixture
        point = points[idx]
        Point_.assert_called_once_with(ser, idx)
        assert point is point_

    def it_raises_on_indexed_access_out_of_range(self):
        points = XyPoints(
            element(
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=3},c:yVal/c:num"
                "Ref/c:numCache/c:ptCount{val=3})"
            )
        )
        with pytest.raises(IndexError):
            points[-1]
        with pytest.raises(IndexError):
            points[3]

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def getitem_fixture(self, request, Point_, point_):
        ser = element(
            "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=3},c:yVal/c:num"
            "Ref/c:numCache/c:ptCount{val=3})"
        )
        points = XyPoints(ser)
        idx = 2
        return points, idx, Point_, ser, point_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Point_(self, request, point_):
        return class_mock(request, "pptx.chart.point.Point", return_value=point_)

    @pytest.fixture
    def point_(self, request):
        return instance_mock(request, Point)


class DescribeBubblePoints(object):
    def it_supports_len(self, len_fixture):
        points, expected_value = len_fixture
        assert len(points) == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:ser", 0),
            ("c:ser/c:bubbleSize/c:numRef/c:numCache/c:ptCount{val=3}", 0),
            (
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=1},c:yVal/c:numRef"
                "/c:numCache/c:ptCount{val=2},c:bubbleSize/c:numRef/c:numCache/c:pt"
                "Count{val=3})",
                1,
            ),
            (
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=3},c:yVal/c:numRef"
                "/c:numCache/c:ptCount{val=3},c:bubbleSize/c:numRef/c:numCache/c:pt"
                "Count{val=3})",
                3,
            ),
        ]
    )
    def len_fixture(self, request):
        ser_cxml, expected_value = request.param
        points = BubblePoints(element(ser_cxml))
        return points, expected_value


class DescribeCategoryPoints(object):
    def it_supports_len(self, len_fixture):
        points, expected_value = len_fixture
        assert len(points) == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:ser", 0),
            ("c:ser/c:cat/c:numRef/c:numCache/c:ptCount{val=42}", 42),
            ("c:ser/c:cat/c:numRef/c:numCache/c:ptCount{val=24}", 24),
        ]
    )
    def len_fixture(self, request):
        ser_cxml, expected_value = request.param
        points = CategoryPoints(element(ser_cxml))
        return points, expected_value


class DescribePoint(object):
    def it_provides_access_to_its_data_label(self, data_label_fixture):
        point, DataLabel_, ser, idx, data_label_ = data_label_fixture
        data_label = point.data_label
        DataLabel_.assert_called_once_with(ser, idx)
        assert data_label is data_label_

    def it_provides_access_to_its_format(self, format_fixture):
        point, ChartFormat_, ser, chart_format_, expected_xml = format_fixture
        chart_format = point.format
        ChartFormat_.assert_called_once_with(ser.xpath('c:dPt[c:idx/@val="42"]')[0])
        assert chart_format is chart_format_
        assert point._element.xml == expected_xml

    def it_provides_access_to_its_marker(self, marker_fixture):
        point, Marker_, dPt, marker_ = marker_fixture
        marker = point.marker
        Marker_.assert_called_once_with(dPt)
        assert marker is marker_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def data_label_fixture(self, DataLabel_, data_label_):
        ser, idx = element("c:ser"), 42
        point = Point(ser, idx)
        return point, DataLabel_, ser, idx, data_label_

    @pytest.fixture(
        params=[
            ("c:ser", "c:ser/c:dPt/c:idx{val=42}"),
            ("c:ser/c:dPt/c:idx{val=42}", "c:ser/c:dPt/c:idx{val=42}"),
            (
                "c:ser/c:dPt/c:idx{val=45}",
                "c:ser/(c:dPt/c:idx{val=45},c:dPt/c:idx{val=42})",
            ),
        ]
    )
    def format_fixture(self, request, ChartFormat_, chart_format_):
        ser_cxml, expected_cxml = request.param
        ser = element(ser_cxml)
        point = Point(ser, 42)
        expected_xml = xml(expected_cxml)
        return point, ChartFormat_, ser, chart_format_, expected_xml

    @pytest.fixture
    def marker_fixture(self, Marker_, marker_):
        ser = element("c:ser/c:dPt/c:idx{val=42}")
        point = Point(ser, 42)
        dPt = ser[0]
        return point, Marker_, dPt, marker_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ChartFormat_(self, request, chart_format_):
        return class_mock(
            request, "pptx.chart.point.ChartFormat", return_value=chart_format_
        )

    @pytest.fixture
    def chart_format_(self, request):
        return instance_mock(request, ChartFormat)

    @pytest.fixture
    def DataLabel_(self, request, data_label_):
        return class_mock(
            request, "pptx.chart.point.DataLabel", return_value=data_label_
        )

    @pytest.fixture
    def data_label_(self, request):
        return instance_mock(request, DataLabel)

    @pytest.fixture
    def Marker_(self, request, marker_):
        return class_mock(request, "pptx.chart.point.Marker", return_value=marker_)

    @pytest.fixture
    def marker_(self, request):
        return instance_mock(request, Marker)


class DescribeXyPoints(object):
    def it_supports_len(self, len_fixture):
        points, expected_value = len_fixture
        assert len(points) == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("c:ser", 0),
            ("c:ser/c:xVal", 0),
            ("c:ser/c:xVal/c:numRef/c:numCache/c:ptCount{val=3}", 0),
            ("c:ser/c:yVal/c:numRef/c:numCache/c:ptCount{val=3}", 0),
            (
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=1},c:yVal/c:numRef"
                "/c:numCache/c:ptCount{val=3})",
                1,
            ),
            (
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=3},c:yVal/c:numRef"
                "/c:numCache/c:ptCount{val=1})",
                1,
            ),
            (
                "c:ser/(c:xVal/c:numRef/c:numCache/c:ptCount{val=3},c:yVal/c:numRef"
                "/c:numCache/c:ptCount{val=3})",
                3,
            ),
        ]
    )
    def len_fixture(self, request):
        ser_cxml, expected_value = request.param
        points = XyPoints(element(ser_cxml))
        return points, expected_value



# encoding: utf-8

"""
XML test data builders for pptx.oxml.dml unit tests
"""

from __future__ import absolute_import, print_function

from ...unitdata import BaseBuilder


class CT_HslColorBuilder(BaseBuilder):
    __tag__ = "a:hslClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("hue", "sat", "lum")


class CT_LinePropertiesBuilder(BaseBuilder):
    __tag__ = "a:ln"
    __nspfxs__ = ("a",)
    __attrs__ = ("w", "cap", "cmpd", "algn")


class CT_PercentageBuilder(BaseBuilder):
    __nspfxs__ = ("a",)
    __attrs__ = ("val",)

    def __init__(self, tag):
        self.__tag__ = tag
        super(CT_PercentageBuilder, self).__init__()


class CT_PresetColorBuilder(BaseBuilder):
    __tag__ = "a:prstClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("val",)


class CT_SolidColorFillPropertiesBuilder(BaseBuilder):
    __tag__ = "a:solidFill"
    __nspfxs__ = ("a",)
    __attrs__ = ()


class CT_SchemeColorBuilder(BaseBuilder):
    __tag__ = "a:schemeClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("val",)


class CT_ScRgbColorBuilder(BaseBuilder):
    __tag__ = "a:scrgbClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("r", "g", "b")


class CT_SRgbColorBuilder(BaseBuilder):
    __tag__ = "a:srgbClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("val",)


class CT_SystemColorBuilder(BaseBuilder):
    __tag__ = "a:sysClr"
    __nspfxs__ = ("a",)
    __attrs__ = ("val", "lastClr")


def a_lumMod():
    return CT_PercentageBuilder("a:lumMod")


def a_lumOff():
    return CT_PercentageBuilder("a:lumOff")


def a_prstClr():
    return CT_PresetColorBuilder()


def a_schemeClr():
    return CT_SchemeColorBuilder()


def a_solidFill():
    return CT_SolidColorFillPropertiesBuilder()


def an_hslClr():
    return CT_HslColorBuilder()


def an_ln():
    return CT_LinePropertiesBuilder()


def an_scrgbClr():
    return CT_ScRgbColorBuilder()


def an_srgbClr():
    return CT_SRgbColorBuilder()


def a_sysClr():
    return CT_SystemColorBuilder()



# encoding: utf-8

"""
Objects shared by pptx modules.
"""

from __future__ import absolute_import, division, print_function, unicode_literals


class ElementProxy(object):
    """
    Base class for lxml element proxy classes. An element proxy class is one
    whose primary responsibilities are fulfilled by manipulating the
    attributes and child elements of an XML element. They are the most common
    type of class in python-pptx other than custom element (oxml) classes.
    """

    __slots__ = ("_element",)

    def __init__(self, element):
        self._element = element

    def __eq__(self, other):
        """
        Return |True| if this proxy object refers to the same oxml element as
        does *other*. ElementProxy objects are value objects and should
        maintain no mutable local state. Equality for proxy objects is
        defined as referring to the same XML element, whether or not they are
        the same proxy object instance.
        """
        if not isinstance(other, ElementProxy):
            return False
        return self._element is other._element

    def __ne__(self, other):
        if not isinstance(other, ElementProxy):
            return True
        return self._element is not other._element

    @property
    def element(self):
        """
        The lxml element proxied by this object.
        """
        return self._element


class ParentedElementProxy(ElementProxy):
    """
    Provides common services for document elements that occur below a part
    but may occasionally require an ancestor object to provide a service,
    such as add or drop a relationship. Provides the :attr:`_parent`
    attribute to subclasses and the public :attr:`parent` read-only property.
    """

    __slots__ = ("_parent",)

    def __init__(self, element, parent):
        super(ParentedElementProxy, self).__init__(element)
        self._parent = parent

    @property
    def parent(self):
        """
        The ancestor proxy object to this one. For example, the parent of
        a shape is generally the |SlideShapes| object that contains it.
        """
        return self._parent

    @property
    def part(self):
        """
        The package part containing this object
        """
        return self._parent.part


class PartElementProxy(ElementProxy):
    """
    Provides common members for proxy objects that wrap the root element of
    a part such as `p:sld`.
    """

    __slots__ = ("_part",)

    def __init__(self, element, part):
        super(PartElementProxy, self).__init__(element)
        self._part = part

    @property
    def part(self):
        """
        The package part containing this object
        """
        return self._part



# encoding: utf-8

"""
Objects shared by modules in the pptx.opc sub-package
"""

from __future__ import absolute_import, print_function, unicode_literals


class CaseInsensitiveDict(dict):
    """
    Mapping type that behaves like dict except that it matches without respect
    to the case of the key. E.g. cid['A'] == cid['a']. Note this is not
    general-purpose, just complete enough to satisfy opc package needs. It
    assumes str keys for example.
    """

    def __contains__(self, key):
        return super(CaseInsensitiveDict, self).__contains__(key.lower())

    def __getitem__(self, key):
        return super(CaseInsensitiveDict, self).__getitem__(key.lower())

    def __setitem__(self, key, value):
        return super(CaseInsensitiveDict, self).__setitem__(key.lower(), value)



# encoding: utf-8

"""lxml custom element class for CT_GraphicalObjectFrame XML element."""

from pptx.oxml import parse_xml
from pptx.oxml.chart.chart import CT_Chart
from pptx.oxml.ns import nsdecls
from pptx.oxml.shapes.shared import BaseShapeElement
from pptx.oxml.simpletypes import XsdBoolean, XsdString
from pptx.oxml.table import CT_Table
from pptx.oxml.xmlchemy import (
    BaseOxmlElement,
    OneAndOnlyOne,
    OptionalAttribute,
    RequiredAttribute,
    ZeroOrOne,
)
from pptx.spec import (
    GRAPHIC_DATA_URI_CHART,
    GRAPHIC_DATA_URI_OLEOBJ,
    GRAPHIC_DATA_URI_TABLE,
)


class CT_GraphicalObject(BaseOxmlElement):
    """
    ``<a:graphic>`` element, which is the container for the reference to or
    definition of the framed graphical object (table, chart, etc.).
    """

    graphicData = OneAndOnlyOne("a:graphicData")

    @property
    def chart(self):
        """
        The ``<c:chart>`` grandchild element, or |None| if not present.
        """
        return self.graphicData.chart


class CT_GraphicalObjectData(BaseShapeElement):
    """
    ``<p:graphicData>`` element, the direct container for a table, a chart,
    or another graphical object.
    """

    chart = ZeroOrOne("c:chart")
    tbl = ZeroOrOne("a:tbl")
    uri = RequiredAttribute("uri", XsdString)

    @property
    def blob_rId(self):
        """Optional "r:id" attribute value of `<p:oleObj>` descendent element.

        This value is `None` when this `p:graphicData` element does not enclose an OLE
        object. This value could also be `None` if an enclosed OLE object does not
        specify this attribute (it is specified optional in the schema) but so far, all
        OLE objects we've encountered specify this value.
        """
        return None if self._oleObj is None else self._oleObj.rId

    @property
    def is_embedded_ole_obj(self):
        """Optional boolean indicating an embedded OLE object.

        Returns `None` when this `p:graphicData` element does not enclose an OLE object.
        `True` indicates an embedded OLE object and `False` indicates a linked OLE
        object.
        """
        return None if self._oleObj is None else self._oleObj.is_embedded

    @property
    def progId(self):
        """Optional str value of "progId" attribute of `<p:oleObj>` descendent.

        This value identifies the "type" of the embedded object in terms of the
        application used to open it.

        This value is `None` when this `p:graphicData` element does not enclose an OLE
        object. This could also be `None` if an enclosed OLE object does not specify
        this attribute (it is specified optional in the schema) but so far, all OLE
        objects we've encountered specify this value.
        """
        return None if self._oleObj is None else self._oleObj.progId

    @property
    def showAsIcon(self):
        """Optional value of "showAsIcon" attribute value of `p:oleObj` descendent.

        This value is `None` when this `p:graphicData` element does not enclose an OLE
        object. It is False when the `showAsIcon` attribute is omitted on the `p:oleObj`
        element.
        """
        return None if self._oleObj is None else self._oleObj.showAsIcon

    @property
    def _oleObj(self):
        """Optional `<p:oleObj>` element contained in this `p:graphicData' element.

        Returns `None` when this graphic-data element does not enclose an OLE object.
        Note that this returns the last `p:oleObj` element found. There can be more
        than one `p:oleObj` element because an `<mc.AlternateContent>` element may
        appear as the child of `p:graphicData` and that alternate-content subtree can
        contain multiple compatibility choices. The last one should suit best for
        reading purposes because it contains the lowest common denominator.
        """
        oleObjs = self.xpath(".//p:oleObj")
        return oleObjs[-1] if oleObjs else None


class CT_GraphicalObjectFrame(BaseShapeElement):
    """
    ``<p:graphicFrame>`` element, which is a container for a table, a chart,
    or another graphical object.
    """

    nvGraphicFramePr = OneAndOnlyOne("p:nvGraphicFramePr")
    xfrm = OneAndOnlyOne("p:xfrm")
    graphic = OneAndOnlyOne("a:graphic")

    @property
    def chart(self):
        """
        The ``<c:chart>`` great-grandchild element, or |None| if not present.
        """
        return self.graphic.chart

    @property
    def chart_rId(self):
        """
        The ``rId`` attribute of the ``<c:chart>`` great-grandchild element,
        or |None| if not present.
        """
        chart = self.chart
        if chart is None:
            return None
        return chart.rId

    def get_or_add_xfrm(self):
        """
        Return the required ``<p:xfrm>`` child element. Overrides version on
        BaseShapeElement.
        """
        return self.xfrm

    @property
    def graphicData(self):
        """`<a:graphicData> grandchild of this graphic-frame element."""
        return self.graphic.graphicData

    @property
    def graphicData_uri(self):
        """str value of `uri` attribute of `<a:graphicData> grandchild."""
        return self.graphic.graphicData.uri

    @property
    def has_oleobj(self):
        """True for graphicFrame containing an OLE object, False otherwise."""
        return self.graphicData.uri == GRAPHIC_DATA_URI_OLEOBJ

    @property
    def is_embedded_ole_obj(self):
        """Optional boolean indicating an embedded OLE object.

        Returns `None` when this `p:graphicFrame` element does not enclose an OLE
        object. `True` indicates an embedded OLE object and `False` indicates a linked
        OLE object.
        """
        return self.graphicData.is_embedded_ole_obj

    @classmethod
    def new_chart_graphicFrame(cls, id_, name, rId, x, y, cx, cy):
        """
        Return a ``<p:graphicFrame>`` element tree populated with a chart
        element.
        """
        graphicFrame = CT_GraphicalObjectFrame.new_graphicFrame(id_, name, x, y, cx, cy)
        graphicData = graphicFrame.graphic.graphicData
        graphicData.uri = GRAPHIC_DATA_URI_CHART
        graphicData.append(CT_Chart.new_chart(rId))
        return graphicFrame

    @classmethod
    def new_graphicFrame(cls, id_, name, x, y, cx, cy):
        """
        Return a new ``<p:graphicFrame>`` element tree suitable for
        containing a table or chart. Note that a graphicFrame element is not
        a valid shape until it contains a graphical object such as a table.
        """
        xml = cls._graphicFrame_tmpl() % (id_, name, x, y, cx, cy)
        graphicFrame = parse_xml(xml)
        return graphicFrame

    @classmethod
    def new_ole_object_graphicFrame(
        cls, id_, name, ole_object_rId, progId, icon_rId, x, y, cx, cy
    ):
        """Return newly-created `<p:graphicFrame>` for embedded OLE-object.

        `ole_object_rId` identifies the relationship to the OLE-object part.

        `progId` is a str identifying the object-type in terms of the application
        (program) used to open it. This becomes an attribute of the same name in the
        `p:oleObj` element.

        `icon_rId` identifies the relationship to an image part used to display the
        OLE-object as an icon (vs. a preview).
        """
        return parse_xml(
            cls._graphicFrame_xml_for_ole_object(
                id_, name, x, y, cx, cy, ole_object_rId, progId, icon_rId
            )
        )

    @classmethod
    def new_table_graphicFrame(cls, id_, name, rows, cols, x, y, cx, cy):
        """
        Return a ``<p:graphicFrame>`` element tree populated with a table
        element.
        """
        graphicFrame = cls.new_graphicFrame(id_, name, x, y, cx, cy)
        graphicFrame.graphic.graphicData.uri = GRAPHIC_DATA_URI_TABLE
        graphicFrame.graphic.graphicData.append(CT_Table.new_tbl(rows, cols, cx, cy))
        return graphicFrame

    @classmethod
    def _graphicFrame_tmpl(cls):
        return (
            "<p:graphicFrame %s>\n"
            "  <p:nvGraphicFramePr>\n"
            '    <p:cNvPr id="%s" name="%s"/>\n'
            "    <p:cNvGraphicFramePr>\n"
            '      <a:graphicFrameLocks noGrp="1"/>\n'
            "    </p:cNvGraphicFramePr>\n"
            "    <p:nvPr/>\n"
            "  </p:nvGraphicFramePr>\n"
            "  <p:xfrm>\n"
            '    <a:off x="%s" y="%s"/>\n'
            '    <a:ext cx="%s" cy="%s"/>\n'
            "  </p:xfrm>\n"
            "  <a:graphic>\n"
            "    <a:graphicData/>\n"
            "  </a:graphic>\n"
            "</p:graphicFrame>"
            % (nsdecls("a", "p"), "%d", "%s", "%d", "%d", "%d", "%d")
        )

    @classmethod
    def _graphicFrame_xml_for_ole_object(
        cls, id_, name, x, y, cx, cy, ole_object_rId, progId, icon_rId
    ):
        """str XML for <a:graphic> element of an embedded OLE-object shape."""
        return (
            "<p:graphicFrame {nsdecls}>\n"
            "  <p:nvGraphicFramePr>\n"
            '    <p:cNvPr id="{id_}" name="{name}"/>\n'
            "    <p:cNvGraphicFramePr>\n"
            '      <a:graphicFrameLocks noGrp="1"/>\n'
            "    </p:cNvGraphicFramePr>\n"
            "    <p:nvPr/>\n"
            "  </p:nvGraphicFramePr>\n"
            "  <p:xfrm>\n"
            '    <a:off x="{x}" y="{y}"/>\n'
            '    <a:ext cx="{cx}" cy="{cy}"/>\n'
            "  </p:xfrm>\n"
            "  <a:graphic>\n"
            "    <a:graphicData"
            '        uri="http://schemas.openxmlformats.org/presentationml/2006/ole">\n'
            '      <p:oleObj showAsIcon="1"'
            '                r:id="{ole_object_rId}"'
            '                imgW="965200"'
            '                imgH="609600"'
            '                progId="{progId}">\n'
            "        <p:embed/>\n"
            "        <p:pic>\n"
            "          <p:nvPicPr>\n"
            '            <p:cNvPr id="0" name=""/>\n'
            "            <p:cNvPicPr/>\n"
            "            <p:nvPr/>\n"
            "          </p:nvPicPr>\n"
            "          <p:blipFill>\n"
            '            <a:blip r:embed="{icon_rId}"/>\n'
            "            <a:stretch>\n"
            "              <a:fillRect/>\n"
            "            </a:stretch>\n"
            "          </p:blipFill>\n"
            "          <p:spPr>\n"
            "            <a:xfrm>\n"
            '              <a:off x="{x}" y="{y}"/>\n'
            '              <a:ext cx="{cx}" cy="{cy}"/>\n'
            "            </a:xfrm>\n"
            '            <a:prstGeom prst="rect">\n'
            "              <a:avLst/>\n"
            "            </a:prstGeom>\n"
            "          </p:spPr>\n"
            "        </p:pic>\n"
            "      </p:oleObj>\n"
            "    </a:graphicData>\n"
            "  </a:graphic>\n"
            "</p:graphicFrame>"
        ).format(
            nsdecls=nsdecls("a", "p", "r"),
            id_=id_,
            name=name,
            x=x,
            y=y,
            cx=cx,
            cy=cy,
            ole_object_rId=ole_object_rId,
            progId=progId,
            icon_rId=icon_rId,
        )


class CT_GraphicalObjectFrameNonVisual(BaseOxmlElement):
    """`<p:nvGraphicFramePr>` element.

    This contains the non-visual properties of a graphic frame, such as name, id, etc.
    """

    cNvPr = OneAndOnlyOne("p:cNvPr")
    nvPr = OneAndOnlyOne("p:nvPr")


class CT_OleObject(BaseOxmlElement):
    """`<p:oleObj>` element, container for an OLE object (e.g. Excel file).

    An OLE object can be either linked or embedded (hence the name).
    """

    progId = OptionalAttribute("progId", XsdString)
    rId = OptionalAttribute("r:id", XsdString)
    showAsIcon = OptionalAttribute("showAsIcon", XsdBoolean, default=False)

    @property
    def is_embedded(self):
        """True when this OLE object is embedded, False when it is linked."""
        return True if len(self.xpath("./p:embed")) > 0 else False



# encoding: utf-8

"""
Parser for Compact XML Expression Language (CXEL) ('see-ex-ell'), a compact
XML specification language I made up that's useful for producing XML element
trees suitable for unit testing.
"""

from __future__ import print_function

from pyparsing import (
    alphas,
    alphanums,
    Combine,
    dblQuotedString,
    delimitedList,
    Forward,
    Group,
    Literal,
    Optional,
    removeQuotes,
    stringEnd,
    Suppress,
    Word,
)

from pptx.oxml import parse_xml
from pptx.oxml.ns import _nsmap as nsmap


# ====================================================================
# api functions
# ====================================================================


def element(cxel_str):
    """
    Return an oxml element parsed from the XML generated from *cxel_str*.
    """
    _xml = xml(cxel_str)
    return parse_xml(_xml)


def xml(cxel_str):
    """Return the XML generated from *cxel_str*."""
    root_node.parseWithTabs()
    root_token = root_node.parseString(cxel_str)
    xml = root_token.element.xml
    return xml


# ====================================================================
# internals
# ====================================================================


def nsdecls(*nspfxs):
    """
    Return a string containing a namespace declaration for each of *nspfxs*,
    in the order they are specified.
    """
    nsdecls = ""
    for nspfx in nspfxs:
        nsdecls += ' xmlns:%s="%s"' % (nspfx, nsmap[nspfx])
    return nsdecls


class Element(object):
    """
    Represents an XML element, having a namespace, tagname, attributes, and
    may contain either text or children (but not both) or may be empty.
    """

    def __init__(self, tagname, attrs, text):
        self._tagname = tagname
        self._attrs = attrs
        self._text = text
        self._children = []
        self._is_root = False

    def __repr__(self):
        """
        Provide a more meaningful repr value for an Element object, one that
        displays the tagname as a simple empty element, e.g. ``<w:pPr/>``.
        """
        return "<%s/>" % self._tagname

    def connect_children(self, child_node_list):
        """
        Make each of the elements appearing in *child_node_list* a child of
        this element.
        """
        for node in child_node_list:
            child = node.element
            self._children.append(child)

    @classmethod
    def from_token(cls, token):
        """
        Return an ``Element`` object constructed from a parser element token.
        """
        tagname = token.tagname
        attrs = [(name, value) for name, value in token.attr_list]
        text = token.text
        return cls(tagname, attrs, text)

    @property
    def is_root(self):
        """
        |True| if this element is the root of the tree and should include the
        namespace prefixes. |False| otherwise.
        """
        return self._is_root

    @is_root.setter
    def is_root(self, value):
        self._is_root = bool(value)

    @property
    def local_nspfxs(self):
        """
        The namespace prefixes local to this element, both on the tagname and
        all of its attributes. An empty string (``''``) is used to represent
        the default namespace for an element tag having no prefix.
        """

        def nspfx(name, is_element=False):
            idx = name.find(":")
            if idx == -1:
                return "" if is_element else None
            return name[:idx]

        nspfxs = [nspfx(self._tagname, True)]
        for name, val in self._attrs:
            pfx = nspfx(name)
            if pfx is None or pfx in nspfxs:
                continue
            nspfxs.append(pfx)
        return nspfxs

    @property
    def nspfxs(self):
        """
        A sequence containing each of the namespace prefixes appearing in
        this tree. Each prefix appears once and only once, and in document
        order.
        """

        def merge(seq, seq_2):
            for item in seq_2:
                if item in seq:
                    continue
                seq.append(item)

        nspfxs = self.local_nspfxs
        for child in self._children:
            merge(nspfxs, child.nspfxs)
        return nspfxs

    @property
    def xml(self):
        """
        The XML corresponding to the tree rooted at this element,
        pretty-printed using 2-spaces indentation at each level and with
        a trailing '\n'.
        """
        return self._xml(indent=0)

    def _xml(self, indent):
        """
        Return a string containing the XML of this element and all its
        children with a starting indent of *indent* spaces.
        """
        self._indent_str = " " * indent
        xml = self._start_tag
        for child in self._children:
            xml += child._xml(indent + 2)
        xml += self._end_tag
        return xml

    @property
    def _start_tag(self):
        """
        The text of the opening tag of this element, including attributes. If
        this is the root element, a namespace declaration for each of the
        namespace prefixes that occur in this tree is added in front of any
        attributes. If this element contains text, that text follows the
        start tag. If not, and this element has no children, an empty tag is
        returned. Otherwise, an opening tag is returned, followed by
        a newline. The tag is indented by this element's indent value in all
        cases.
        """
        _nsdecls = nsdecls(*self.nspfxs) if self.is_root else ""
        tag = "%s<%s%s" % (self._indent_str, self._tagname, _nsdecls)
        for attr in self._attrs:
            name, value = attr
            tag += ' %s="%s"' % (name, value)
        if self._text:
            tag += ">%s" % self._text
        elif self._children:
            tag += ">\n"
        else:
            tag += "/>\n"
        return tag

    @property
    def _end_tag(self):
        """
        The text of the closing tag of this element, if there is one. If the
        element contains text, no leading indentation is included.
        """
        if self._text:
            tag = "</%s>\n" % self._tagname
        elif self._children:
            tag = "%s</%s>\n" % (self._indent_str, self._tagname)
        else:
            tag = ""
        return tag


# ====================================================================
# parser
# ====================================================================

# parse actions ----------------------------------


def connect_node_children(s, loc, tokens):
    node = tokens[0]
    node.element.connect_children(node.child_node_list)


def connect_root_node_children(root_node):
    root_node.element.connect_children(root_node.child_node_list)
    root_node.element.is_root = True


def grammar():
    # terminals ----------------------------------
    colon = Literal(":")
    equal = Suppress("=")
    slash = Suppress("/")
    open_paren = Suppress("(")
    close_paren = Suppress(")")
    open_brace = Suppress("{")
    close_brace = Suppress("}")

    # np:tagName ---------------------------------
    nspfx = Word(alphas)
    local_name = Word(alphanums)
    tagname = Combine(nspfx + colon + local_name)

    # np:attr_name=attr_val ----------------------
    attr_name = Word(alphas + ":")
    attr_val = Word(alphanums + " %-./:_")
    attr_def = Group(attr_name + equal + attr_val)
    attr_list = open_brace + delimitedList(attr_def) + close_brace

    text = dblQuotedString.setParseAction(removeQuotes)

    # w:jc{val=right} ----------------------------
    element = (
        tagname("tagname")
        + Group(Optional(attr_list))("attr_list")
        + Optional(text, default="")("text")
    ).setParseAction(Element.from_token)

    child_node_list = Forward()

    node = Group(
        element("element") + Group(Optional(slash + child_node_list))("child_node_list")
    ).setParseAction(connect_node_children)

    child_node_list << (open_paren + delimitedList(node) + close_paren | node)

    root_node = (
        element("element")
        + Group(Optional(slash + child_node_list))("child_node_list")
        + stringEnd
    ).setParseAction(connect_root_node_children)

    return root_node


root_node = grammar()
